# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dashboard_app', '0008_imagechartfilter_is_all_tests_included'),
    ]

    operations = [
        migrations.AlterField(
            model_name='testresult',
            name='measurement',
            field=models.CharField(blank=True, max_length=512, help_text="Arbitrary value that was measured as a part of this test.", null=True, verbose_name="Measurement"),
        ),
    ]



# Copyright (C) 2017 Linaro Limited
#
# Author: Dean Arnold <dean.arnold@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.action import Pipeline
from lava_dispatcher.actions.boot import AutoLoginAction, BootAction, OverlayUnpack
from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
from lava_dispatcher.logical import Boot
from lava_dispatcher.power import ResetDevice
from lava_dispatcher.connections.serial import ConnectDevice
from lava_dispatcher.shell import ExpectShellSession


class Minimal(Boot):

    compatibility = 1

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = MinimalBoot()
        self.action.section = self.action_type
        self.action.job = self.job
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        if "minimal" not in device["actions"]["boot"]["methods"]:
            return False, '"minimal" was not in device configuration boot methods'
        if "method" not in parameters:
            return False, '"method" was not in parameters'
        if parameters["method"] != "minimal":
            return False, '"method" was not "minimal"'
        return True, "accepted"


class MinimalBoot(BootAction):

    name = "minimal-boot"
    description = "connect and reset device"
    summary = "connect and reset device"

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.internal_pipeline.add_action(ConnectDevice())
        self.internal_pipeline.add_action(ResetDevice())
        if self.has_prompts(parameters):
            self.internal_pipeline.add_action(AutoLoginAction())
            if self.test_has_shell(parameters):
                self.internal_pipeline.add_action(ExpectShellSession())
                if "transfer_overlay" in parameters:
                    self.internal_pipeline.add_action(OverlayUnpack())
                self.internal_pipeline.add_action(ExportDeviceEnvironment())

    def run(self, connection, max_end_time):
        connection = self.get_namespace_data(
            action="shared", label="shared", key="connection", deepcopy=False
        )
        connection = super().run(connection, max_end_time)
        self.set_namespace_data(
            action="shared", label="shared", key="connection", value=connection
        )
        return connection



import os
import yaml
import logging
import tempfile
from lava_scheduler_app.utils import split_multinode_yaml
from lava_scheduler_app.dbutils import match_vlan_interface
from lava_scheduler_app.models import TestJob, Tag
from lava_scheduler_app.tests.test_submission import TestCaseWithFactory
from lava_scheduler_app.tests.test_pipeline import YamlFactory
from lava_dispatcher.device import NewDevice
from lava_dispatcher.parser import JobParser
from lava_dispatcher.protocols.vland import VlandProtocol
from lava_dispatcher.protocols.multinode import MultinodeProtocol

# pylint does not like TestCaseWithFactory
# pylint: disable=too-many-ancestors,no-self-use,too-many-statements
# pylint: disable=superfluous-parens,too-many-locals


class VlandFactory(YamlFactory):
    def __init__(self):
        super().__init__()
        self.bbb1 = None
        self.cubie1 = None
        self.bbb_type = None

    def setUp(self):  # pylint: disable=invalid-name
        self.bbb_type = self.make_device_type(name="bbb")
        self.cubie_type = self.make_device_type(name="cubietruck")
        self.bbb1 = self.make_device(self.bbb_type, hostname="bbb-01")
        self.cubie1 = self.make_device(self.cubie_type, hostname="cubie1")

    def make_vland_job(self, **kw):
        sample_job_file = os.path.join(
            os.path.dirname(__file__), "sample_jobs", "bbb-cubie-vlan-group.yaml"
        )
        with open(sample_job_file, "r") as test_support:
            data = yaml.safe_load(test_support)
        data.update(kw)
        return data


class TestVlandSplit(TestCaseWithFactory):
    """
    Test the splitting of lava-vland data from submission YAML
    Same tests as test_submission but converted to use and look for YAML.
    """

    def setUp(self):
        super().setUp()
        self.factory = VlandFactory()

    def test_split_vland(self):
        target_group = "unit-test-only"
        job_dict = split_multinode_yaml(self.factory.make_vland_job(), target_group)
        self.assertEqual(len(job_dict), 2)
        roles = job_dict.keys()
        self.assertEqual({"server", "client"}, set(roles))
        for role in roles:
            self.assertEqual(len(job_dict[role]), 1)  # count = 1
        client_job = job_dict["client"][0]
        server_job = job_dict["server"][0]
        self.assertIn("lava-multinode", client_job["protocols"])
        self.assertIn("lava-multinode", server_job["protocols"])
        self.assertIn("lava-vland", client_job["protocols"])
        self.assertIn("lava-vland", server_job["protocols"])
        client_vlan = client_job["protocols"]["lava-vland"]
        server_vlan = server_job["protocols"]["lava-vland"]
        self.assertIn("vlan_one", client_vlan)
        self.assertIn("vlan_two", server_vlan)
        self.assertEqual(["RJ45", "10M"], list(client_vlan.values())[0]["tags"])
        self.assertEqual(["RJ45", "100M"], list(server_vlan.values())[0]["tags"])


class TestVlandDevices(TestCaseWithFactory):
    """
    Test the matching of vland device requirements with submission YAML
    """

    def setUp(self):
        super().setUp()
        self.factory = VlandFactory()
        self.factory.setUp()
        logger = logging.getLogger("lava-master")
        logger.disabled = True

    def test_match_devices_without_map(self):
        """
        Without a map, there is no support for knowing which interfaces to
        put onto a VLAN, so these devices cannot be assigned to a VLAN testjob
        See http://localhost/static/docs/v2/vland.html#vland-and-interface-tags-in-lava
        """
        self.bbb3 = self.factory.make_device(self.factory.bbb_type, hostname="bbb-03")
        self.cubie2 = self.factory.make_device(
            self.factory.cubie_type, hostname="cubie2"
        )
        devices = [self.bbb3, self.cubie2]
        self.factory.ensure_tag("usb-eth")
        self.factory.ensure_tag("sata")
        self.factory.bbb1.tags.set(Tag.objects.filter(name="usb-eth"))
        self.factory.cubie1.tags.set(Tag.objects.filter(name="sata"))
        user = self.factory.make_user()
        sample_job_file = os.path.join(
            os.path.dirname(__file__), "sample_jobs", "bbb-cubie-vlan-group.yaml"
        )
        with open(sample_job_file, "r") as test_support:
            data = yaml.safe_load(test_support)
        vlan_job = TestJob.from_yaml_and_user(yaml.dump(data), user)
        assignments = {}
        for job in vlan_job:
            self.assertFalse(
                match_vlan_interface(self.bbb3, yaml.safe_load(job.definition))
            )
            self.assertFalse(
                match_vlan_interface(self.cubie2, yaml.safe_load(job.definition))
            )

    def test_jinja_template(self):
        yaml_data = self.factory.bbb1.load_configuration()
        self.assertIn("parameters", yaml_data)
        self.assertIn("interfaces", yaml_data["parameters"])
        self.assertIn("bootm", yaml_data["parameters"])
        self.assertIn("bootz", yaml_data["parameters"])
        self.assertIn("actions", yaml_data)
        self.assertIn("eth0", yaml_data["parameters"]["interfaces"])
        self.assertIn("eth1", yaml_data["parameters"]["interfaces"])
        self.assertIn("sysfs", yaml_data["parameters"]["interfaces"]["eth0"])
        self.assertEqual(
            "/sys/devices/pci0000:00/0000:00:1c.1/0000:03:00.0/net/eth1",
            yaml_data["parameters"]["interfaces"]["eth1"]["sysfs"],
        )


class TestVlandProtocolSplit(TestCaseWithFactory):
    """
    Test the handling of protocols in dispatcher after splitting the YAML
    """

    def setUp(self):
        super().setUp()
        self.factory = VlandFactory()
        self.factory.setUp()

    def test_job_protocols(self):
        self.factory.ensure_tag("usb-eth")
        self.factory.ensure_tag("sata")
        self.factory.bbb1.tags.set(Tag.objects.filter(name="usb-eth"))
        self.factory.cubie1.tags.set(Tag.objects.filter(name="sata"))
        target_group = "unit-test-only"
        job_dict = split_multinode_yaml(self.factory.make_vland_job(), target_group)
        client_job = job_dict["client"][0]
        client_handle, client_file_name = tempfile.mkstemp()
        yaml.dump(client_job, open(client_file_name, "w"))
        # YAML device file, as required by lava-dispatch --target
        device_yaml_file = os.path.realpath(
            os.path.join(os.path.dirname(__file__), "devices", "bbb-01.yaml")
        )
        self.assertTrue(os.path.exists(device_yaml_file))
        parser = JobParser()
        bbb_device = NewDevice(device_yaml_file)
        with open(client_file_name) as sample_job_data:
            bbb_job = parser.parse(sample_job_data, bbb_device, 4212, None, "")
        os.close(client_handle)
        os.unlink(client_file_name)
        self.assertIn("protocols", bbb_job.parameters)
        self.assertIn(VlandProtocol.name, bbb_job.parameters["protocols"])
        self.assertIn(MultinodeProtocol.name, bbb_job.parameters["protocols"])



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2018 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.core.management.base import BaseCommand

from lava_scheduler_app.models import TestJob
from lava_scheduler_app.utils import mkdir

import os
import time


class Command(BaseCommand):
    help = "Move job outputs into the new location"

    def add_arguments(self, parser):
        parser.add_argument(
            "--dry-run",
            default=False,
            action="store_true",
            help="Do not move any data, simulate the output",
        )
        parser.add_argument(
            "--slow",
            default=False,
            action="store_true",
            help="Be nice with the system by sleeping regularly",
        )

    def handle(self, *_, **options):
        base_dir = "/var/lib/lava-server/default/media/job-output/"
        len_base_dir = len(base_dir)
        jobs = TestJob.objects.all().order_by("id")

        self.stdout.write("Browsing all jobs")
        start = 0
        while True:
            count = 0
            for job in jobs[start : start + 100]:
                count += 1
                old_path = os.path.join(
                    settings.MEDIA_ROOT, "job-output", "job-%s" % job.id
                )
                date_path = os.path.join(
                    settings.MEDIA_ROOT,
                    "job-output",
                    "%02d" % job.submit_time.year,
                    "%02d" % job.submit_time.month,
                    "%02d" % job.submit_time.day,
                    str(job.id),
                )
                if not os.path.exists(old_path):
                    self.stdout.write("* %d skip" % job.id)
                    continue

                self.stdout.write(
                    "* %d {%s => %s}"
                    % (job.id, old_path[len_base_dir:], date_path[len_base_dir:])
                )
                if not options["dry_run"]:
                    mkdir(os.path.dirname(date_path))
                    if not os.path.exists(old_path):
                        self.stdout.write("  -> no output directory")
                        continue
                    os.rename(old_path, date_path)
            start += count
            if count == 0:
                break
            if options["slow"]:
                self.stdout.write("sleeping 2s...")
                time.sleep(2)



# -*- coding: utf-8 -*-
# Copyright (C) 2015-2018 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#         Stevan Radakovic <stevan.radakovic@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

"""
URL mappings for the LAVA Results application
"""
from django.conf.urls import url
from lava_results_app.views import (
    add_bug_link,
    delete_bug_link,
    get_bug_links_json,
    index,
    metadata_export,
    suite,
    suite_csv_stream,
    suite_csv,
    suite_yaml,
    suite_testcase_count,
    testcase,
    testcase_yaml,
    testjob,
    testjob_csv,
    testjob_yaml,
    testjob_yaml_summary,
    testset,
)
from lava_results_app.views.query.views import (
    get_query_group_names,
    get_query_names,
    query_list,
    query_add,
    query_add_condition,
    query_add_group,
    query_copy,
    query_custom,
    query_delete,
    query_detail,
    query_display,
    query_edit,
    query_edit_condition,
    query_export,
    query_export_custom,
    query_group_list,
    query_refresh,
    query_remove_condition,
    query_omit_result,
    query_include_result,
    query_select_group,
    query_toggle_published,
)
from lava_results_app.views.chart.views import (
    get_chart_group_names,
    chart_list,
    chart_add,
    chart_add_group,
    chart_custom,
    chart_delete,
    chart_detail,
    chart_display,
    chart_edit,
    chart_group_list,
    chart_select_group,
    chart_toggle_published,
    chart_query_add,
    chart_query_edit,
    chart_query_remove,
    chart_query_order_update,
    chart_omit_result,
    settings_update,
)

urlpatterns = [
    url(r"^$", index, name="lava_results"),
    url(r"^query$", query_list, name="lava.results.query_list"),
    url(r"^query/\+add$", query_add, name="lava.results.query_add"),
    url(r"^query/\+custom$", query_custom, name="lava.results.query_custom"),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)$",
        query_display,
        name="lava.results.query_display",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+detail$",
        query_detail,
        name="lava.results.query_detail",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+edit$",
        query_edit,
        name="lava.results.query_edit",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+delete$",
        query_delete,
        name="lava.results.query_delete",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+export$",
        query_export,
        name="lava.results.query_export",
    ),
    url(
        r"^query/\+export-custom$",
        query_export_custom,
        name="lava.results.query_export_custom",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+toggle-published$",
        query_toggle_published,
        name="lava.results.query_toggle_published",
    ),
    url(r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+copy$", query_copy),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+refresh$",
        query_refresh,
        name="lava.results.query_refresh",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+add-condition$",
        query_add_condition,
        name="lava.results.query_add_condition",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+remove-condition$",
        query_remove_condition,
        name="lava.results.query_remove_condition",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+edit-condition$",
        query_edit_condition,
        name="lava.results.query_edit_condition",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+add-group$",
        query_add_group,
        name="query_add_group",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+select-group$",
        query_select_group,
        name="query_select_group",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+omit-result$",
        query_omit_result,
        name="lava.results.query_omit_result",
    ),
    url(
        r"^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+include-result$",
        query_include_result,
        name="lava.results.query_include_result",
    ),
    url(r"^query/get-query-groups$", query_group_list, name="query_group_list"),
    url(
        r"^query/\+get-group-names$",
        get_query_group_names,
        name="get_query_group_names",
    ),
    url(
        r"^query/\+get-query-names$",
        get_query_names,
        name="lava.results.get_query_names",
    ),
    url(r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)$", testjob, name="lava.results.testjob"),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/csv$",
        testjob_csv,
        name="lava.results.testjob_csv",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/yaml$",
        testjob_yaml,
        name="lava.results.testjob_yaml",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/yaml_summary$",
        testjob_yaml_summary,
        name="lava.results.testjob_yaml_summary",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/metadata$",
        metadata_export,
        name="lava.results.job.metadata",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)$",
        suite,
        name="lava.results.suite",
    ),
    url(r"^chart$", chart_list, name="lava.results.chart_list"),
    url(r"^chart/\+add$", chart_add, name="lava.results.chart_add"),
    url(r"^chart/\+custom$", chart_custom, name="lava.results.chart_custom"),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)$",
        chart_display,
        name="lava.results.chart_display",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+detail$",
        chart_detail,
        name="lava.results.chart_detail",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+edit$",
        chart_edit,
        name="lava.results.chart_edit",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+delete$",
        chart_delete,
        name="lava.results.chart_delete",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+toggle-published$",
        chart_toggle_published,
        name="lava.results.chart_toggle_published",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+chart-query-add$",
        chart_query_add,
        name="lava.results.chart_query_add",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+chart-query-remove$",
        chart_query_remove,
        name="lava.results.chart_query_remove",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+chart-query-edit$",
        chart_query_edit,
        name="lava.results.chart_query_edit",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+add-group$",
        chart_add_group,
        name="chart_add_group",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+select-group$",
        chart_select_group,
        name="chart_select_group",
    ),
    url(r"^chart/\+get-chart-groups$", chart_group_list, name="chart_group_list"),
    url(
        r"^chart/\+get-group-names$",
        get_chart_group_names,
        name="get_chart_group_names",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+settings-update$",
        settings_update,
        name="chart_settings_update",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/\+chart-query-order-update$",
        chart_query_order_update,
        name="chart_query_order_update",
    ),
    url(
        r"^chart/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/(?P<result_id>\d+)/\+omit-result$",
        chart_omit_result,
        name="lava.results.chart_omit_result",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/(?P<ts>[-_a-zA-Z0-9.]+)/(?P<case>[-_a-zA-Z0-9.+]+)$",
        testset,
        name="lava.results.testset",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+\.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/csv$",
        suite_csv,
        name="lava.results.suite_csv",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/stream/csv$",
        suite_csv_stream,
        name="lava.results.suite_csv_stream",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/yaml$",
        suite_yaml,
        name="lava.results.suite_yaml",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/\+testcase-count$",
        suite_testcase_count,
        name="lava.results.suite_testcase_count",
    ),
    url(r"^testcase/(?P<case_id>[0-9]+)$", testcase, name="lava.results.testcase"),
    url(
        r"^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/(?P<case_id>[0-9]+)$",
        testcase,
        name="lava.results.testcase",
    ),
    url(
        r"^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/(?P<case_id>[-_a-zA-Z0-9.\(\)+]+)$",
        testcase,
        name="lava.results.testcase",
    ),
    url(
        r"^testcase/(?P<pk>[0-9]+)/yaml$",
        testcase_yaml,
        name="lava.results.testcase_yaml",
    ),
    url(
        r"^get-bug-links-json$",
        get_bug_links_json,
        name="lava.results.get_bug_links_json",
    ),
    url(r"^add-bug-link$", add_bug_link, name="lava.results.add_bug_link"),
    url(r"^delete-bug-link$", delete_bug_link, name="lava.results.delete_bug_link"),
]



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2018 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#         Senthil Kumaran <senthil.kumaran@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

from functools import wraps
from simplejson import JSONDecodeError
import yaml
import sys
import xmlrpc.client
from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.db.models import Count, Q
from django.db import transaction
from linaro_django_xmlrpc.models import ExposedAPI
from lava_scheduler_app.models import (
    Device,
    DeviceType,
    JSONDataError,
    DevicesUnavailableException,
    TestJob,
)
from lava_scheduler_app.views import get_restricted_job
from lava_scheduler_app.dbutils import (
    device_type_summary,
    testjob_submission,
    active_device_types,
)
from lava_scheduler_app.schema import (
    validate_submission,
    validate_device,
    SubmissionException,
)

# functions need to be members to be exposed in the API
# pylint: disable=no-self-use

# to make a function visible in the API, it must be a member of SchedulerAPI
# pylint: disable=no-self-use


def check_perm(perm):
    """ decorator to check that the caller has the given permission """

    def decorator(f):
        @wraps(f)
        def wrapper(self, *args, **kwargs):
            self._authenticate()
            if not self.user.has_perm(perm):
                raise xmlrpc.client.Fault(
                    403,
                    "User '%s' is missing permission %s ." % (self.user.username, perm),
                )
            return f(self, *args, **kwargs)

        return wrapper

    return decorator


def build_device_status_display(state, health):
    if state == Device.STATE_IDLE:
        if health in [Device.HEALTH_GOOD, Device.HEALTH_UNKNOWN]:
            return "idle"
        elif health == Device.HEALTH_RETIRED:
            return "retired"
        else:
            return "offline"
    elif state == Device.STATE_RESERVED:
        return "reserved"
    else:
        return "running"


class SchedulerAPI(ExposedAPI):
    def submit_job(self, job_data):
        """
        Name
        ----
        `submit_job` (`job_data`)

        Description
        -----------
        Submit the given job data which is in LAVA job JSON or YAML format as a
        new job to LAVA scheduler.

        Arguments
        ---------
        `job_data`: string
            Job JSON or YAML string.

        Return value
        ------------
        This function returns an XML-RPC integer which is the newly created
        job's id, provided the user is authenticated with an username and token.
        If the job is a multinode job, this function returns the list of created
        job IDs.
        """
        self._authenticate()
        if not self.user.has_perm("lava_scheduler_app.add_testjob"):
            raise xmlrpc.client.Fault(
                403,
                "Permission denied.  User %r does not have the "
                "'lava_scheduler_app.add_testjob' permission.  Contact "
                "the administrators." % self.user.username,
            )
        try:
            job = testjob_submission(job_data, self.user)
        except SubmissionException as exc:
            raise xmlrpc.client.Fault(400, "Problem with submitted job data: %s" % exc)
        # FIXME: json error is not needed anymore
        except (JSONDataError, JSONDecodeError, ValueError) as exc:
            raise xmlrpc.client.Fault(400, "Decoding job submission failed: %s." % exc)
        except (Device.DoesNotExist, DeviceType.DoesNotExist):
            raise xmlrpc.client.Fault(404, "Specified device or device type not found.")
        except DevicesUnavailableException as exc:
            raise xmlrpc.client.Fault(400, "Device unavailable: %s" % str(exc))
        if isinstance(job, list):
            return [j.sub_id for j in job]
        else:
            return job.id

    def resubmit_job(self, job_id):
        """
        Name
        ----
        `resubmit_job` (`job_id`)

        Description
        -----------
        Resubmit the given job reffered by its id.

        Arguments
        ---------
        `job_id`: string
            The job's id which should be re-submitted.

        Return value
        ------------
        This function returns an XML-RPC integer which is the newly created
        job's id,  provided the user is authenticated with an username and
        token.
        """
        self._authenticate()
        if not self.user.has_perm("lava_scheduler_app.add_testjob"):
            raise xmlrpc.client.Fault(
                403,
                "Permission denied.  User %r does not have the "
                "'lava_scheduler_app.add_testjob' permission.  Contact "
                "the administrators." % self.user.username,
            )
        try:
            job = get_restricted_job(self.user, job_id)
        except TestJob.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified job not found.")

        if job.is_multinode:
            return self.submit_job(job.multinode_definition)
        else:
            return self.submit_job(job.definition)

    def cancel_job(self, job_id):
        """
        Name
        ----
        `cancel_job` (`job_id`)

        Description
        -----------
        Cancel the given job reffered by its id.

        Arguments
        ---------
        `job_id`: string
            Job id which should be canceled.

        Return value
        ------------
        None. The user should be authenticated with an username and token.
        """
        self._authenticate()
        if not job_id:
            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")

        with transaction.atomic():
            try:
                job = get_restricted_job(self.user, job_id, for_update=True)
            except PermissionDenied:
                raise xmlrpc.client.Fault(
                    401, "Permission denied for user to job %s" % job_id
                )
            except TestJob.DoesNotExist:
                raise xmlrpc.client.Fault(404, "Specified job not found.")

            if job.state in [TestJob.STATE_CANCELING, TestJob.STATE_FINISHED]:
                # Don't do anything for jobs that ended already
                return True
            if not job.can_cancel(self.user):
                raise xmlrpc.client.Fault(403, "Permission denied.")

            if job.is_multinode:
                multinode_jobs = TestJob.objects.select_for_update().filter(
                    target_group=job.target_group
                )
                for multinode_job in multinode_jobs:
                    multinode_job.go_state_canceling()
                    multinode_job.save()
            else:
                job.go_state_canceling()
                job.save()
        return True

    def validate_yaml(self, yaml_string):
        """
        Name
        ----
        validate_yaml (yaml_job_data)

        Description
        -----------
        Validate the supplied pipeline YAML against the submission schema.

        Note: this does not validate the job itself, just the YAML against the
        submission schema. A job which validates against the schema can still be
        an invalid job for the dispatcher and such jobs will be accepted as Submitted,
        scheduled and then marked as Incomplete with a failure comment. Full validation
        only happens after a device has been assigned to the Submitted job.

        Arguments
        ---------
        'yaml_job_data': string

        Return value
        ------------
        Raises an Exception if the yaml_job_data is invalid.
        """
        try:
            # YAML can parse JSON as YAML, JSON cannot parse YAML at all
            yaml_data = yaml.safe_load(yaml_string)
        except yaml.YAMLError as exc:
            raise xmlrpc.client.Fault(400, "Decoding job submission failed: %s." % exc)
        try:
            # validate against the submission schema.
            validate_submission(yaml_data)  # raises SubmissionException if invalid.
        except SubmissionException as exc:
            raise xmlrpc.client.Fault(400, "Invalid YAML submission: %s" % exc)

    def job_output(self, job_id, offset=0):
        """
        Name
        ----
        `job_output` (`job_id`, `offset=0`)

        Description
        -----------
        Get the output of given job id.

        Arguments
        ---------
        `job_id`: string
            Job id for which the output is required.
        `offset`: integer
            Offset from which to start reading the output file specified in bytes.
            It defaults to 0.

        Return value
        ------------
        This function returns an XML-RPC binary data of output file, provided
        the user is authenticated with an username and token.
        """
        self._authenticate()
        if not job_id:
            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
        try:
            job = get_restricted_job(self.user, job_id)
        except PermissionDenied:
            raise xmlrpc.client.Fault(
                401, "Permission denied for user to job %s" % job_id
            )
        except TestJob.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified job not found.")

        output_file = job.output_file()
        if output_file:
            output_file.seek(offset)
            return xmlrpc.client.Binary(output_file.read().encode("UTF-8"))
        else:
            raise xmlrpc.client.Fault(404, "Job output not found.")

    def all_devices(self):
        """
        Name
        ----
        `all_devices` ()

        Description
        -----------
        Get all the available devices with their state and type information.

        Arguments
        ---------
        None

        Return value
        ------------
        This function returns an XML-RPC array in which each item is a list of
        device hostname, device type, device state, current running job id and
        if device is pipeline. For example:

        [['panda01', 'panda', 'running', 'good', 164, False], ['qemu01', 'qemu', 'idle', 'unknwon', None, True]]
        """

        devices_list = []
        for dev in Device.objects.exclude(health=Device.HEALTH_RETIRED):
            if not dev.is_visible_to(self.user):
                continue
            devices_list.append(dev)

        return [
            [
                dev.hostname,
                dev.device_type.name,
                build_device_status_display(dev.state, dev.health),
                dev.current_job().pk if dev.current_job() else None,
                True,
            ]
            for dev in devices_list
        ]

    def all_device_types(self):
        """
        Name
        ----
        `all_device_types` ()

        Description
        -----------
        Get all the available device types with their state and count
        information.

        Arguments
        ---------
        None

        Return value
        ------------
        This function returns an XML-RPC array in which each item is a dict
        which contains name (device type), idle, busy, offline counts.
        For example:

        [{'idle': 1, 'busy': 0, 'name': 'panda', 'offline': 0},
        {'idle': 1, 'busy': 0, 'name': 'qemu', 'offline': 0}]
        """

        device_type_names = []
        all_device_types = []
        keys = ["busy", "idle", "offline"]

        for dev_type in DeviceType.objects.all():
            if not dev_type.some_devices_visible_to(self.user):
                continue
            device_type_names.append(dev_type.name)

        device_types = device_type_summary(device_type_names)

        for dev_type in device_types:
            device_type = {"name": dev_type["device_type"]}
            for key in keys:
                device_type[key] = dev_type[key]
            all_device_types.append(device_type)

        return all_device_types

    def get_recent_jobs_for_device_type(
        self, device_type, count=1, restrict_to_user=False
    ):
        """
        Name
        ----

        `get_recent_jobs_for_device_type` (`device_type`, `count=1`, `restrict_to_user=False`)

        Description
        -----------
        Get details of recently finished jobs for a given device_type. Limits the list
        to test jobs submitted by the user making the query if restrict_to_user is set to
        True. Get only the most recent job by default, but count can be set higher to
        get for example the last 10 jobs.

        Arguments
        ---------
        `device_type`: string
            Name of the device_type for which you want the jobs
        `count`: integer (Optional, default=1)
            Number of last jobs you want
        `restrict_to_user`: boolean (Optional, default=False)
            Fetch only the jobs submitted by the user making the query if set to True

        Return value
        ------------
        This function returns a list of dictionaries, which correspond to the
        list of recently finished jobs informations (Complete or Incomplete)
        for this device, ordered from youngest to oldest.

        [
            {
                'description': 'ramdisk health check',
                'id': 359828,
                'status': 'Complete',
                'device': 'black01'
            },
            {
                'description': 'standard ARMMP NFS',
                'id': 359827
                'status': 'Incomplete',
                'device': 'black02'
            }
        ]
        """
        if not device_type:
            raise xmlrpc.client.Fault(
                400, "Bad request: device_type was not specified."
            )
        if count < 0:
            raise xmlrpc.client.Fault(400, "Bad request: count must not be negative.")
        try:
            dt = DeviceType.objects.get(name=device_type, display=True)
        except Device.DoesNotExist:
            raise xmlrpc.client.Fault(
                404, "DeviceType '%s' was not found." % device_type
            )

        if not dt.some_devices_visible_to(self.user):
            raise xmlrpc.client.Fault(
                403,
                "DeviceType '%s' not available to user '%s'."
                % (device_type, self.user),
            )
        job_qs = (
            TestJob.objects.filter(state=TestJob.STATE_FINISHED)
            .filter(requested_device_type=dt)
            .order_by("-id")
        )
        if restrict_to_user:
            job_qs = job_qs.filter(submitter=self.user)
        job_list = []
        for job in job_qs.all()[:count]:
            hostname = ""
            if job.actual_device:
                hostname = job.actual_device.hostname
            job_dict = {
                "id": job.id,
                "description": job.description,
                "status": job.get_legacy_status_display(),
                "device": hostname,
            }
            if not job.can_view(self.user):
                job_dict["id"] = None
            job_list.append(job_dict)
        return job_list

    def get_recent_jobs_for_device(self, device, count=1, restrict_to_user=False):
        """
        Name
        ----

        `get_recent_jobs_for_device` (`device`, `count=1`, `restrict_to_user=False`)

        Description
        -----------
        Get details of recently finished jobs for a given device. Limits the list
        to test jobs submitted by the user making the query if restrict_to_user is set to
        True. Get only the most recent job by default, but count can be set higher to
        get for example the last 10 jobs.

        Arguments
        ---------
        `device`: string
            Name of the device for which you want the jobs
        `count`: integer (Optional, default=1)
            Number of last jobs you want
        `restrict_to_user`: boolean (Optional, default=False)
            Fetch only the jobs submitted by the user making the query if set to True

        Return value
        ------------
        This function returns a list of dictionaries, which correspond to the
        list of recently finished jobs informations (Complete or Incomplete)
        for this device, ordered from youngest to oldest.

        [
            {
                'description': 'mainline--armada-370-db--multi_v7_defconfig--network',
                'id': 359828,
                'status': 'Complete'
            },
            {
                'description': 'mainline--armada-370-db--multi_v7_defconfig--sata',
                'id': 359827
                'status': 'Incomplete'
            }
        ]
        """
        if not device:
            raise xmlrpc.client.Fault(400, "Bad request: device was not specified.")
        if count < 0:
            raise xmlrpc.client.Fault(400, "Bad request: count must not be negative.")
        try:
            device_obj = Device.objects.get(hostname=device)
        except Device.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Device '%s' was not found." % device)

        if not device_obj.is_visible_to(self.user):
            raise xmlrpc.client.Fault(
                403, "Device '%s' not available to user '%s'." % (device, self.user)
            )
        job_qs = (
            TestJob.objects.filter(state=TestJob.STATE_FINISHED)
            .filter(actual_device=device_obj)
            .order_by("-id")
        )
        if restrict_to_user:
            job_qs = job_qs.filter(submitter=self.user)
        job_list = []
        for job in job_qs.all()[:count]:
            job_dict = {
                "id": job.id,
                "description": job.description,
                "status": job.get_legacy_status_display(),
            }
            if not job.can_view(self.user):
                job_dict["id"] = None
            job_list.append(job_dict)
        return job_list

    def get_device_type_by_alias(self, alias):
        """
        Name
        ----

        `get_device_type_by_alias` (`alias`)

        Description
        -----------
        Get the matching device-type(s) for the specified alias. It is
        possible that more than one device-type can be returned, depending
        on local admin configuration. An alias can be used to provide the
        link between the device-type name and the Device Tree name.
        It is possible for multiple device-types to have the same alias
        (to assist in transitions and migrations).
        The specified alias string can be a partial match, returning all
        device-types which have an alias name containing the requested
        string.

        Arguments
        ---------
        `alias`: string
            Name of the alias to lookup

        Return value
        ------------
        This function returns a dictionary containing the alias as the key
        and a list of device-types which use that alias as the value. If the
        specified alias does not match any device-type, the dictionary contains
        an empty list for the alias key.

        {'apq8016-sbc': ['dragonboard410c']}
        {'ompa4-panda': ['panda', 'panda-es']}
        """

        aliases = DeviceType.objects.filter(aliases__name__contains=alias)
        return {alias: [device_type.name for device_type in aliases]}

    def get_device_status(self, hostname):
        """
        Name
        ----
        `get_device_status` (`hostname`)

        Description
        -----------
        Get status, running job, date from which it is offline of the given
        device and the user who put it offline.

        Arguments
        ---------
        `hostname`: string
            Name of the device for which the status is asked.

        Return value
        ------------
        This function returns an XML-RPC dictionary which contains hostname,
        status, date from which the device is offline if the device is offline,
        the user who put the device offline if the device is offline and the
        job id of the running job.
        The device has to be visible to the user who requested device's status.

        Note that offline_since and offline_by can be empty strings if the device
        status is manually changed by an administrator in the database or from
        the admin site of LAVA even if device's status is offline.
        """

        if not hostname:
            raise xmlrpc.client.Fault(400, "Bad request: Hostname was not specified.")
        try:
            device = Device.objects.get(hostname=hostname)
        except Device.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)

        device_dict = {}
        if device.is_visible_to(self.user):
            device_dict["hostname"] = device.hostname
            device_dict["status"] = build_device_status_display(
                device.state, device.health
            )
            device_dict["job"] = None
            device_dict["offline_since"] = None
            device_dict["offline_by"] = None
            device_dict["is_pipeline"] = True

            current_job = device.current_job()
            if current_job is not None:
                device_dict["job"] = current_job.pk
        else:
            raise xmlrpc.client.Fault(
                403, "Permission denied for user to access %s information." % hostname
            )
        return device_dict

    def put_into_maintenance_mode(self, hostname, reason, notify=None):
        """
        Name
        ----
        `put_into_maintenance_mode` (`hostname`, `reason`, `notify`)

        Description
        -----------
        Put the given device in maintenance mode with the given reason and optionally
        notify the given mail address when the job has finished.

        Arguments
        ---------
        `hostname`: string
            Name of the device to put into maintenance mode.
        `reason`: string
            The reason given to justify putting the device into maintenance mode.
        `notify`: string
            Email address of the user to notify when the job has finished. Can be
            omitted.

        Return value
        ------------
        None. The user should be authenticated with a username and token and has
        sufficient permission.
        """

        self._authenticate()
        if not hostname:
            raise xmlrpc.client.Fault(400, "Bad request: Hostname was not specified.")
        if not reason:
            raise xmlrpc.client.Fault(400, "Bad request: Reason was not specified.")
        with transaction.atomic():
            try:
                device = Device.objects.select_for_update().get(hostname=hostname)
            except Device.DoesNotExist:
                raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)
            if device.can_admin(self.user):
                device.health = Device.HEALTH_MAINTENANCE
                device.save()
            else:
                raise xmlrpc.client.Fault(
                    403,
                    "Permission denied for user to put %s into maintenance mode."
                    % hostname,
                )

    def put_into_online_mode(self, hostname, reason, skip_health_check=False):
        """
        Name
        ----
        `put_into_online_mode` (`hostname`, `reason`, `skip_health_check`)

        Description
        -----------
        Put the given device into online mode with the given reason ans skip health
        check if asked.

        Arguments
        ---------
        `hostname`: string
            Name of the device to put into online mode.
        `reason`: string
            The reason given to justify putting the device into online mode.
        `skip_health_check`: boolean
            Skip health check when putting the board into online mode. If
            omitted, health check is not skipped by default.

        Return value
        ------------
        None. The user should be authenticated with a username and token and has
        sufficient permission.
        """

        self._authenticate()
        if not hostname:
            raise xmlrpc.client.Fault(400, "Bad request: Hostname was not specified.")
        if not reason:
            raise xmlrpc.client.Fault(400, "Bad request: Reason was not specified.")
        with transaction.atomic():
            try:
                device = Device.objects.select_for_update().get(hostname=hostname)
            except Device.DoesNotExist:
                raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)
            if device.can_admin(self.user):
                device.health = Device.HEALTH_UNKNOWN
                device.save()
            else:
                raise xmlrpc.client.Fault(
                    403,
                    "Permission denied for user to put %s into online mode." % hostname,
                )

    def pending_jobs_by_device_type(self, all=False):
        """
        Name
        ----
        `pending_jobs_by_device_type` ()

        Description
        -----------
        Get number of pending jobs in each device type.
        Private test jobs and hidden device types are
        excluded, except for authenticated superusers.

        Arguments
        ---------
        `all`: boolean - include retired devices and undisplayed device-types in the listing.

        Return value
        ------------
        This function returns a dict where the key is the device type and
        the value is the number of jobs pending in that device type.
        For example:

        {'qemu': 0, 'panda': 3}
        """

        pending_jobs_by_device = {}

        jobs_res = TestJob.objects.filter(state=TestJob.STATE_SUBMITTED)
        jobs_res = jobs_res.exclude(requested_device_type_id__isnull=True)

        if not self.user or not self.user.is_superuser:
            jobs_res = jobs_res.filter(is_public=True)

        jobs_res = jobs_res.values_list("requested_device_type_id")
        jobs_res = jobs_res.annotate(pending_jobs=(Count("id")))

        jobs = {}
        jobs_hash = dict(jobs_res)
        for job in jobs_hash:
            jobs[job] = jobs_hash[job]
        pending_jobs_by_device.update(jobs)

        # Get rest of the devices and put number of pending jobs as 0.
        if all:
            device_types = DeviceType.objects.all()
        else:
            device_types = active_device_types()

        if not self.user or not self.user.is_superuser:
            device_types = device_types.filter(owners_only=False)

        for device_type in device_types.values_list("name", flat=True):
            if device_type not in pending_jobs_by_device:
                pending_jobs_by_device[device_type] = 0

        return pending_jobs_by_device

    def job_details(self, job_id):
        """
        Name
        ----
        `job_details` (`job_id`)

        Description
        -----------
        Get the details of given job id.

        Arguments
        ---------
        `job_id`: string
            Job id for which the output is required.

        Return value
        ------------
        This function returns an XML-RPC structures of job details, provided
        the user is authenticated with an username and token.

        The elements available in XML-RPC structure include:
        _state, submitter_id, is_pipeline, id, failure_comment,
        multinode_definition, user_id, priority, _actual_device_cache,
        original_definition, status, health_check, description,
        admin_notifications, start_time, target_group, visibility,
        pipeline_compatibility, submit_time, is_public, _old_status,
        actual_device_id, definition, sub_id, requested_device_type_id,
        end_time, group_id, absolute_url, submitter_username
        """
        self._authenticate()
        if not job_id:
            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
        try:
            job = get_restricted_job(self.user, job_id)
            job.status = job.get_legacy_status_display()
            job.state = job.get_state_display()
            job.health = job.get_health_display()
            job.submitter_username = job.submitter.username
            job.absolute_url = job.get_absolute_url()
            job.is_pipeline = True
        except PermissionDenied:
            raise xmlrpc.client.Fault(
                401, "Permission denied for user to job %s" % job_id
            )
        except TestJob.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified job not found.")

        return job

    def job_status(self, job_id):
        """
        Name
        ----
        DEPRECATED - use `job_health` or `job_state` instead.

        `job_status` (`job_id`)

        Description
        -----------
        Get the status of given job id.

        Arguments
        ---------
        `job_id`: string
            Job id for which the output is required.

        Return value
        ------------
        This function returns an XML-RPC structures of job status with the
        following fields.
        The user is authenticated with an username and token.

        `job_status`: string
        ['Submitted'|'Running'|'Complete'|'Incomplete'|'Canceled'|'Canceling']

        `bundle_sha1`: string
        The sha1 hash code of the bundle, if it existed. Otherwise it will be
        an empty string. (LAVA V1 only)
        """
        self._authenticate()
        if not job_id:
            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
        try:
            job = get_restricted_job(self.user, job_id)
        except PermissionDenied:
            raise xmlrpc.client.Fault(
                401, "Permission denied for user to job %s" % job_id
            )
        except TestJob.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified job not found.")

        job_status = {"job_id": job.id}

        if job.is_multinode:
            job_status.update({"sub_id": job.sub_id})

        job_status.update(
            {"job_status": job.get_legacy_status_display(), "bundle_sha1": ""}
        )
        return job_status

    def job_list_status(self, job_id_list):
        """
        Name
        ----
        DEPRECATED - use `job_health` or `job_state` instead.

        job_list_status ([job_id, job_id, job_id])

        Description
        -----------
        Get the status of a list of job ids.

        Arguments
        ---------
        `job_id_list`: list
            List of job ids for which the output is required.
            For multinode jobs specify the job sub_id as a float
            in the XML-RPC call:
            job_list_status([1, 2, 3,1, 5])

        Return value
        ------------
        The user needs to be authenticated with an username and token.

        This function returns an XML-RPC structure of job status with the
        following content.

        `job_status`: string
        {ID: ['Submitted'|'Running'|'Complete'|'Incomplete'|'Canceled'|'Canceling']}

        If the user is not able to view one of the specified jobs, that entry
        will be omitted.

        """
        self._authenticate()
        job_status = {}
        # optimise the query for a long list instead of using the
        # convenience handlers
        if not isinstance(job_id_list, list):
            raise xmlrpc.client.Fault(400, "Bad request: needs to be a list")
        if not all(isinstance(chk, (float, int)) for chk in job_id_list):
            raise xmlrpc.client.Fault(
                400, "Bad request: needs to be a list of integers or floats"
            )
        jobs = TestJob.objects.filter(
            Q(id__in=job_id_list) | Q(sub_id__in=job_id_list)
        ).select_related("actual_device", "requested_device_type")
        for job in jobs:
            device_type = job.requested_device_type
            if (
                not job.can_view(self.user)
                or not job.is_accessible_by(self.user)
                and not self.user.is_superuser
            ):
                continue
            if device_type and device_type.owners_only:
                # do the more expensive check second and only for a hidden device type
                if not device_type.some_devices_visible_to(self.user):
                    continue
            job_status[str(job.display_id)] = job.get_legacy_status_display()
        return job_status

    def job_health(self, job_id):
        """
        Name
        ----
        `job_health` (`job_id`)

        Description
        -----------
        Get the health of given job id.

        Arguments
        ---------
        `job_id`: string
            Job id for which the output is required.

        Return value
        ------------
        This function returns an XML-RPC structures of job health with the
        following fields.
        The user is authenticated with an username and token.

        `job_health`: string
        ['Unknown'|'Complete'|'Incomplete'|'Canceled']
        """
        self._authenticate()
        if not job_id:
            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
        try:
            job = get_restricted_job(self.user, job_id)
        except PermissionDenied:
            raise xmlrpc.client.Fault(
                401, "Permission denied for user to job %s" % job_id
            )
        except TestJob.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified job not found.")

        job_health = {"job_id": job.id, "job_health": job.get_health_display()}

        if job.is_multinode:
            job_health.update({"sub_id": job.sub_id})

        return job_health

    def job_state(self, job_id):
        """
        Name
        ----
        `job_state` (`job_id`)

        Description
        -----------
        Get the state of given job id.

        Arguments
        ---------
        `job_id`: string
            Job id for which the output is required.

        Return value
        ------------
        This function returns an XML-RPC structures of job state with the
        following fields.
        The user is authenticated with an username and token.

        `job_state`: string
        ['Submitted'|'Scheduling'|'Scheduled'|'Running'|'Canceling'|'Finished']
        """
        self._authenticate()
        if not job_id:
            raise xmlrpc.client.Fault(400, "Bad request: TestJob id was not specified.")
        try:
            job = get_restricted_job(self.user, job_id)
        except PermissionDenied:
            raise xmlrpc.client.Fault(
                401, "Permission denied for user to job %s" % job_id
            )
        except TestJob.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified job not found.")

        job_state = {"job_id": job.id, "job_state": job.get_state_display()}

        if job.is_multinode:
            job_state.update({"sub_id": job.sub_id})

        return job_state

    def all_jobs(self):
        """
        Name
        ----
        `all_jobs` ()

        Description
        -----------
        Get submitted or running jobs.

        Arguments
        ---------
        None

        Return value
        ------------
        This function returns a XML-RPC array of submitted and running jobs with their status and
        actual device for running jobs and requested device or device type for submitted jobs and
        job sub_id for multinode jobs.
        For example:

        [[73, 'multinode-job', 'submitted', None, 'kvm', '72.1'],
        [72, 'multinode-job', 'submitted', None, 'kvm', '72.0'],
        [71, 'test-job', 'running', 'kvm01', None, None]]
        """

        jobs = TestJob.objects.exclude(state=TestJob.STATE_FINISHED).order_by("-id")
        jobs_list = [
            [
                job.id,
                job.description,
                job.get_legacy_status_display().lower(),
                job.actual_device,
                job.requested_device_type,
                job.sub_id,
            ]
            for job in jobs
        ]

        return jobs_list

    def get_pipeline_device_config(self, device_hostname):
        """
        Name
        ----
        DEPRECATED - use `get_device_config` instead.

        `get_pipeline_device_config` (`device_hostname`)

        Description
        -----------
        Get the pipeline device configuration for given device hostname.

        Arguments
        ---------
        `device_hostname`: string
            Device hostname for which the configuration is required.

        Return value
        ------------
        This function returns an XML-RPC binary data of output file.
        """
        return self.get_device_config(device_hostname)

    def get_device_config(self, device_hostname, context=None):
        """
        New in api_version 2 - see system.api_version()

        Name
        ----
        `get_device_config` (`device_hostname`, context=None)

        Description
        -----------
        Get the device configuration for given device hostname.

        Arguments
        ---------
        `device_hostname`: string
            Device hostname for which the configuration is required.

        Some device templates need a context specified when processing the
        device-type template. This can be specified as a YAML string:

        `get_device_config` `('qemu01', '{arch: amd64}')`

        Return value
        ------------
        This function returns an XML-RPC binary data of output file.
        """
        if not device_hostname:
            raise xmlrpc.client.Fault(
                400, "Bad request: Device hostname was not specified."
            )

        job_ctx = None
        if context is not None:
            try:
                job_ctx = yaml.safe_load(context)
            except yaml.YAMLError as exc:
                raise xmlrpc.client.Fault(
                    400, "Job context '%s' is not valid. %s" % (context, exc)
                )
        try:
            device = Device.objects.get(hostname=device_hostname)
        except Device.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Specified device was not found.")

        config = device.load_configuration(job_ctx=job_ctx, output_format="yaml")

        # validate against the device schema
        validate_device(yaml.safe_load(config))

        return xmlrpc.client.Binary(config.encode("UTF-8"))

    def import_device_dictionary(self, hostname, jinja_str):
        """
        Name
        ----
        `import_device_dictionary` (`device_hostname`, `jinja_string`)

        Description
        -----------
        [superuser only]
        Import or update the device dictionary key value store for a
        pipeline device.

        Arguments
        ---------
        `device_hostname`: string
            Device hostname to update.
        `jinja_str`: string
            Device configuration as Jinja2

        Return value
        ------------
        This function returns an XML-RPC binary data of output file.
        """
        self._authenticate()
        if not self.user.is_superuser:
            raise xmlrpc.client.Fault(
                403, "User '%s' is not superuser." % self.user.username
            )
        try:
            device = Device.objects.get(hostname=hostname)
        except DeviceType.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)
        if not device.save_configuration(jinja_str):
            raise xmlrpc.client.Fault(
                400, "Unable to store the configuration for %s on disk" % hostname
            )

        return "Device dictionary updated for %s" % hostname

    def export_device_dictionary(self, hostname):
        """
        Name
        ----
        `export_device_dictionary` (`device_hostname`)

        Description
        -----------
        [superuser only]
        Export the device dictionary key value store for a
        pipeline device.

        See also get_pipeline_device_config

        Arguments
        ---------
        `device_hostname`: string
            Device hostname to update.

        Return value
        ------------
        This function returns an XML-RPC binary data of output file.
        """
        self._authenticate()
        if not self.user.is_superuser:
            raise xmlrpc.client.Fault(
                403, "User '%s' is not superuser." % self.user.username
            )
        try:
            device = Device.objects.get(hostname=hostname)
        except DeviceType.DoesNotExist:
            raise xmlrpc.client.Fault(404, "Device '%s' was not found." % hostname)
        device_dict = device.load_configuration(output_format="raw")
        if not device_dict:
            raise xmlrpc.client.Fault(
                404, "Device '%s' does not have a device dictionary" % hostname
            )

        return xmlrpc.client.Binary(device_dict.encode("UTF-8"))

    def validate_pipeline_devices(self, name=None):
        """
        Name
        ----
        `validate_pipeline_device` [`name`]

        Description
        -----------
        Validate that the device dictionary and device-type template
        together create a valid YAML file which matches the pipeline
        device schema.
        Retired devices are ignored.

        See also get_pipeline_device_config

        Arguments
        ---------
        `name`: string
            Can be device hostname or device type name.
        If name is specified, method will search for either a matching device
        hostname or matching device type name in which case it will only
        validate that(those) device(s).
        If not specified, this method will validate all non-retired devices
        in the system.

        Return value
        ------------
        This function returns an XML-RPC structure of results with the
        following fields.

        `device_hostname`: {'Valid': null}
        or
        `device_hostname`: {'Invalid': message}
        `

        """
        if not name:
            devices = Device.objects.exclude(health=Device.HEALTH_RETIRED)
        else:
            devices = Device.objects.exclude(health=Device.HEALTH_RETIRED).filter(
                device_type__name=name
            )
            if not devices:
                devices = Device.objects.exclude(health=Device.HEALTH_RETIRED).filter(
                    hostname=name
                )
        if not devices and name:
            raise xmlrpc.client.Fault(
                404, "No devices found with hostname or device type name %s" % name
            )
        if not devices and not name:
            raise xmlrpc.client.Fault(404, "No pipeline device found on this instance.")
        results = {}
        for device in devices:
            key = str(device.hostname)
            config = device.load_configuration(output_format="yaml")
            if config is None:
                results[key] = {"Invalid": "Missing device dictionary"}
                continue
            try:
                # validate against the device schema
                validate_device(yaml.safe_load(config))
            except SubmissionException as exc:
                results[key] = {"Invalid": exc}
                continue
            results[key] = {"Valid": None}
        return xmlrpc.client.Binary(yaml.dump(results).encode("UTF-8"))

    def get_publisher_event_socket(self):
        """
        Name
        ----
        `get_publisher_event_socket`

        Return value
        ------------
        This function exposes the EVENT_SOCKET from the settings file which is
        used for the lava-publisher daemon.
        """
        return settings.EVENT_SOCKET



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-06-16 14:50
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("lava_scheduler_app", "0018_notifications"),
    ]

    operations = [
        migrations.CreateModel(
            name="ExtendedUser",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "irc_handle",
                    models.CharField(
                        blank=True,
                        default=None,
                        max_length=40,
                        null=True,
                        verbose_name="IRC handle",
                    ),
                ),
                (
                    "irc_server",
                    models.CharField(
                        blank=True,
                        default=None,
                        max_length=40,
                        null=True,
                        verbose_name="IRC server",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        )
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2018 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=superfluous-parens,no-self-use

from django.core.management.base import BaseCommand, CommandError, CommandParser

from django.contrib.auth.models import User, Group, Permission


class Command(BaseCommand):
    help = "Manage users"

    def add_arguments(self, parser):
        cmd = self

        class SubParser(CommandParser):
            """
            Sub-parsers constructor that mimic Django constructor.
            See http://stackoverflow.com/a/37414551
            """

            def __init__(self, **kwargs):
                super().__init__(cmd, **kwargs)

        sub = parser.add_subparsers(
            dest="sub_command", help="Sub commands", parser_class=SubParser
        )
        sub.required = True

        # "add" sub-command
        add_parser = sub.add_parser("add", help="Add a group")
        add_parser.add_argument("name", help="Name of the Django group")
        add_parser.add_argument(
            "--username",
            default=None,
            dest="username",
            help="Username of a user to add to the new group",
        )
        add_parser.add_argument(
            "--submitting",
            default=False,
            action="store_true",
            dest="submitting",
            help="Give users in this group permission to submit test jobs.",
        )

        # "update" sub-command
        update_parser = sub.add_parser("update", help="Update a group")
        update_parser.add_argument("name", help="Name of the Django group")
        update_parser.add_argument(
            "--username", dest="username", help="Username of a user to add to the group"
        )
        update_parser.add_argument(
            "--submitting",
            default=False,
            dest="submitting",
            action="store_true",
            help="Give users in this group permission to submit test jobs.",
        )

        # "list" sub-command
        list_parser = sub.add_parser("list", help="List groups")
        list_parser.add_argument(
            "--verbose",
            dest="verbose",
            default=False,
            action="store_true",
            help="Show all permissions for listed groups.",
        )

    def handle(self, *args, **options):
        """ Forward to the right sub-handler """
        if options["sub_command"] == "add":
            self.handle_add(options)
        if options["sub_command"] == "update":
            self.handle_update(options)
        elif options["sub_command"] == "list":
            self.handle_list(options["verbose"])

    def handle_update(self, options):
        """ Update an existing group """
        name = options["name"]
        group, created = Group.objects.get_or_create(name=name)
        if created:
            raise CommandError("Group '%s' does not already exist!" % name)
        submit = Permission.objects.filter(name="Can cancel or resubmit test jobs")
        if options["username"]:
            try:
                user = User.objects.get(username=options["username"])
            except User.DoesNotExist:
                raise CommandError("Unable to find user '%s'" % name)
            print("Adding user '%s' to group '%s'" % (user.username, group.name))
            group.user_set.add(user)
        if options["submitting"]:
            for perm in submit:
                print("Adding permission %s to group %s" % (perm, group.name))
                group.permissions.add(perm)

    def handle_add(self, options):
        """ Create a new group """
        name = options["name"]
        group, created = Group.objects.get_or_create(name=name)
        if not created:
            raise CommandError("Group '%s' already exists!" % name)

        submit = Permission.objects.filter(name="Can cancel or resubmit test jobs")
        if options["username"]:
            try:
                user = User.objects.get(username=options["username"])
            except User.DoesNotExist:
                raise CommandError("Unable to find user '%s'" % name)
            print("Adding user '%s' to group '%s'" % (user.username, group.name))
            group.user_set.add(user)
        if options["submitting"]:
            for perm in submit:
                print("Adding permission %s to group %s" % (perm, group.name))
                group.permissions.add(perm)

    def handle_list(self, verbose=False):
        """ List groups with permissions """
        groups = Group.objects.all().order_by("name")

        self.stdout.write("List of groups:")
        for group in groups:
            out = "* %s" % group.name
            if verbose:
                out = "%s (%s)" % (out, [perm.name for perm in group.permissions.all()])
            self.stdout.write(out)



# -*- coding: utf-8 -*-
# Copyright (C) 2015-2018 Linaro Limited
#
# Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

import ldap

from django.contrib.auth.models import User
from django.core.management.base import BaseCommand, CommandError
from lava_scheduler_app.utils import get_ldap_user_properties


class Command(BaseCommand):
    help = "Add given username from the configured LDAP server."

    def add_arguments(self, parser):
        parser.add_argument("--username", type=str, help="Username to be added.")
        parser.add_argument(
            "--superuser",
            action="store_true",
            dest="superuser",
            default=False,
            help="User added will be made as superuser.",
        )

    def handle(self, *args, **options):
        username = options["username"]
        if username is None:
            raise CommandError("Username not specified.")

        try:
            user_properties = get_ldap_user_properties(username)
            if user_properties is None:
                self.stderr.write('LDAP user "%s" properties incomplete' % username)
                return
        except ldap.NO_SUCH_OBJECT:
            self.stderr.write("User %s does not exist in LDAP" % username)
            return

        try:
            user = User.objects.get(username=username)
            self.stderr.write('User "%s" exists, not overwriting' % username)
        except User.DoesNotExist:
            user = User.objects.create(username=username)
            user.email = user_properties.get("mail", "")
            user.last_name = user_properties.get("sn", "")
            user.first_name = user_properties.get("given_name", "")
            superuser_msg = ""
            if options["superuser"]:
                user.is_staff = True
                user.is_superuser = True
                superuser_msg = "with superuser status"
            user.save()
            self.stdout.write('User "%s" added %s' % (username, superuser_msg))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  version.py
#
#  Copyright 2014 Neil Williams <codehelp@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import subprocess  # nosec - internal
import os
import sys


# pylint: disable=superfluous-parens,too-many-locals


def version_tag():
    """
    Parses the git status to determine if this is a git tag
    or a developer commit and builds a version string combining
    the two. If there is no git directory, relies on this being
    a directory created from the tarball created by setup.py when
    it uses this script and retrieves the original version string
    from that.
    :return: a version string based on the tag and short hash
    """
    args = ["git", "describe"]
    if len(sys.argv) == 2:
        if sys.argv[1] != "sdist":
            args.append(sys.argv[1])
    if os.path.exists("./.git/"):
        pattern = re.compile(r"(?P<tag>.+)\.(?P<commits>\d+)\.g(?P<hash>[abcdef\d]+)")
        describe = (
            subprocess.check_output(args)  # nosec - internal
            .strip()
            .decode("utf-8")
            .replace("-", ".")
        )
        m = pattern.match(describe)
        if m is None:
            return describe
        else:
            d = m.groupdict()
            return "%s.%04d.g%s" % (d["tag"], int(d["commits"]), d["hash"])
    if os.path.exists("debian/changelog"):
        return (
            subprocess.check_output(  # nosec - internal
                ("dpkg-parsechangelog", "--show-field", "Version")
            )
            .strip()
            .decode("utf-8")
            .split("-")[0]
        )


def main():
    print(version_tag())
    return 0


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
# Copyright (C) 2013-2018 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#         Senthil Kumaran <senthil.kumaran@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

import contextlib
import copy
import errno
import ldap
import os
import subprocess  # nosec verified

from django.contrib.sites.models import Site
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from lava_scheduler_app.schema import SubmissionException

DEFAULT_IRC_SERVER = "irc.freenode.net"
DEFAULT_IRC_PORT = 6667

SERVICE_UNKNOWN_ERROR = "service not known"
NO_SUCH_NICK_ERROR = "No such nick/channel"


class IRCSendError(Exception):
    """Global IRC error."""


class IRCServerIncorrectError(IRCSendError):
    """Error raised when IRC server name is erroneous."""


class IRCHandleNotFoundError(IRCSendError):
    """Error raised when user handle is not found on specific server."""


def get_domain():
    domain = "???"
    with contextlib.suppress(Site.DoesNotExist, ImproperlyConfigured):
        site = Site.objects.get_current()
        domain = site.domain

    return domain


def is_member(user, group):
    return user.groups.filter(name="%s" % group).exists()


def _split_multinode_vland(submission, jobs):

    for role, _ in jobs.items():
        # populate the lava-vland protocol metadata
        if len(jobs[role]) != 1:
            raise SubmissionException(
                "vland protocol only supports one device per role."
            )
        jobs[role][0]["protocols"].update(
            {"lava-vland": submission["protocols"]["lava-vland"][role]}
        )
    return jobs


def split_multinode_yaml(
    submission, target_group
):  # pylint: disable=too-many-branches,too-many-locals,too-many-statements
    """
    Handles the lava-multinode protocol requirements.
    Uses the multinode protocol requirements to generate as many YAML
    snippets as are required to create TestJobs for the multinode submission.
    Each multinode YAML submission is only split once for all roles and all sub jobs.
    parameters:
      submission - the dictionary of the submission YAML.
      device_dict - the dictionary mapping device hostnames to roles
      target_group - the uuid of the multinode group
    return:
      None on error or a dictionary of job roles.
      key: role
      value: list of jobs to be created for that role.
     """
    # the list of devices cannot be definite here, only after devices have been reserved

    # FIXME: needs a Protocol base class in the server and protocol-specific split handlers

    copies = [
        "job_name",
        "timeouts",
        "priority",
        "visibility",
        "notify",
        "metadata",
        "reboot_to_fastboot",
    ]
    skip = ["role", "roles"]
    scheduling = [
        "device_type",
        "connection",
        "host_role",
        "context",
    ]  # top level values to be preserved
    maps = ["count"]  # elements to be matched but not listed at top level.

    roles = {}
    actions = {}
    subid = 0

    # FIXME: check structure using a schema

    role_data = submission["protocols"]["lava-multinode"]["roles"]
    group_size = sum(
        [
            role_data[count]["count"]
            for count in role_data
            if "count" in role_data[count]
        ]
    )

    # populate the lava-multinode protocol metadata
    for role, value in submission["protocols"]["lava-multinode"]["roles"].items():
        roles[role] = {}
        for item in copies:
            if item in submission:
                roles[role][item] = submission[item]
        for name in maps:
            if name in value:
                roles[role][name] = value[name]
        for name in scheduling:
            if name in value:
                roles[role][name] = value[name]
        tags = set(value) - set(maps) - set(scheduling)
        params = {
            "target_group": target_group,
            "role": role,
            "group_size": group_size,
            "sub_id": subid,
        }
        if "essential" in value:
            params["essential"] = value
        for tag in tags:
            params[tag] = value[tag]
        roles[role].update({"protocols": {"lava-multinode": params}})
        subid += 1

    # split the submission based on the roles specified for the actions, retaining order.
    for role in roles:
        for action in submission["actions"]:
            for key, value in action.items():
                try:
                    value["role"]
                except (KeyError, TypeError):
                    raise SubmissionException(
                        "Invalid YAML - Did not find a role in action '%s', check for consistent use of whitespace indents."
                        % action.keys()[0]
                    )
                if role in value["role"]:
                    actions.setdefault(role, {"actions": []})
                    actions[role]["actions"].append(
                        {copy.deepcopy(key): copy.deepcopy(value)}
                    )

    # add other parameters from the lava-multinode protocol
    for key, value in submission["protocols"]["lava-multinode"].items():
        if key in skip:
            continue
        for role in roles:
            roles[role]["protocols"]["lava-multinode"][key] = value

    # set the role for each action to the role of the job instead of the original list..
    for role in actions:
        for action in actions[role]["actions"]:
            for key, value in action.items():
                value["role"] = role

    # jobs dictionary lists the jobs per role,
    jobs = {}
    # check the count of the host_roles
    check_count = None
    for role in roles:
        if "host_role" in roles[role]:
            check_count = roles[role]["host_role"]
    for role in roles:
        if role == check_count:
            if roles[role]["count"] != 1:
                raise SubmissionException(
                    "The count for a role designated as a host_role must be 1."
                )
    sub_id_count = 0
    for role in roles:
        jobs[role] = []
        for sub in range(0, roles[role]["count"]):
            job = {}
            job.update(actions[role])
            job.update(roles[role])
            # only here do multiple jobs for the same role differ
            params = job["protocols"]["lava-multinode"]
            params.update({"sub_id": sub_id_count})
            job["protocols"]["lava-multinode"].update(params)
            del params
            for item in maps:
                if item in job:
                    del job[item]
            jobs[role].append(copy.deepcopy(job))
            sub_id_count += 1

    # populate the lava-vland protocol metadata
    if "lava-vland" in submission["protocols"]:
        _split_multinode_vland(submission, jobs)

    # populate the lava-lxc protocol data
    if "lava-lxc" in submission["protocols"]:
        for role, _ in jobs.items():
            if role not in submission["protocols"]["lava-lxc"]:
                continue
            # populate the lava-lxc protocol metadata
            for job in jobs[role]:
                job["protocols"].update(
                    {"lava-lxc": submission["protocols"]["lava-lxc"][role]}
                )

    return jobs


def mkdir(path):
    try:
        os.makedirs(path, mode=0o755)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def send_irc_notification(
    nick, recipient, message, server=DEFAULT_IRC_SERVER, port=DEFAULT_IRC_PORT
):
    """
    Sends private IRC msg with netcat.
    parameters:
      server - the IRC server where the recipient is.
      port - server port used for the communication.
      nick - nick that sends the message.
      recipient - recipient handle.
      message - message content.
    raise:
      If there is an error, raise an exception and pass stderr message.
    """

    netcat_cmd = (
        "echo -e 'NICK %s\nUSER %s 8 * %s\nPRIVMSG %s :%s\nQUIT\n' | nc -i 5 -q 15 %s %s"
        % (nick, nick, nick, recipient, message, server, port)
    )

    proc = subprocess.Popen(  # nosec managed.
        ["/bin/bash", "-c", netcat_cmd],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    if proc.stderr:
        with proc.stderr:
            for line in iter(proc.stderr.readline, b""):
                if SERVICE_UNKNOWN_ERROR in line:
                    raise IRCServerIncorrectError(line)
                else:
                    raise IRCSendError(line)

    with proc.stdout:
        for line in iter(proc.stdout.readline, b""):
            if NO_SUCH_NICK_ERROR in line:
                raise IRCHandleNotFoundError(line)
    proc.wait()


def get_ldap_user_properties(ldap_user):
    """Searches LDAP based on the parameters in settings.conf and returns LDAP
    user properties as a dictionary, eg:

    {uid: 'senthil.kumaran',
     mail: 'senthil.kumaran@linaro.org',
     sn: 'Kumaran',
     given_name: 'Senthil'
    }

    If given ldap_user does not exist, then raise ldap.NO_SUCH_OBJECT
    """
    server_uri = settings.AUTH_LDAP_SERVER_URI
    bind_dn = settings.AUTH_LDAP_BIND_DN
    bind_password = settings.AUTH_LDAP_BIND_PASSWORD
    user_dn_template = settings.AUTH_LDAP_USER_DN_TEMPLATE
    user_search = settings.AUTH_LDAP_USER_SEARCH

    search_scope = ldap.SCOPE_SUBTREE
    attributes = ["uid", "givenName", "sn", "mail"]
    search_filter = "cn=*"

    if user_dn_template:
        user_dn = user_dn_template % {"user": ldap_user}
    if user_search is not None:
        from django_auth_ldap.config import LDAPSearch

        user_dn = user_search.base_dn
        search_filter = user_search.filterstr % {"user": ldap_user}

    user_properties = {}
    if server_uri is not None:
        conn = ldap.initialize(server_uri)
        if user_dn:
            conn.simple_bind_s(bind_dn, bind_password)
            try:
                result = conn.search_s(user_dn, search_scope, search_filter, attributes)
                if len(result) == 1:
                    result_type, result_data = result[0]
                    user_properties["uid"] = result_data.get("uid", [None])[0]
                    user_properties["mail"] = result_data.get("mail", [None])[0]
                    user_properties["sn"] = result_data.get("sn", [None])[0]
                    user_properties["given_name"] = result_data.get(
                        "givenName", [None]
                    )[0]
                    return user_properties
            except ldap.NO_SUCH_OBJECT:
                raise



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-23 14:17
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0023_update_aliases")]

    operations = [
        migrations.AlterField(
            model_name="device",
            name="physical_owner",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="physicalowner",
                to=settings.AUTH_USER_MODEL,
                verbose_name="User with physical access",
            ),
        ),
        migrations.AlterField(
            model_name="device",
            name="worker_host",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="lava_scheduler_app.Worker",
                verbose_name="Worker Host",
            ),
        ),
        migrations.AlterField(
            model_name="devicetype",
            name="architecture",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="device_types",
                to="lava_scheduler_app.Architecture",
            ),
        ),
        migrations.AlterField(
            model_name="devicetype",
            name="bits",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="device_types",
                to="lava_scheduler_app.BitWidth",
            ),
        ),
        migrations.AlterField(
            model_name="devicetype",
            name="processor",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="device_types",
                to="lava_scheduler_app.ProcessorFamily",
            ),
        ),
    ]



# Copyright (C) 2014 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

# List just the subclasses supported for this base strategy
# imported by the parser to populate the list of subclasses.

from lava_dispatcher.action import Pipeline
from lava_common.exceptions import ConfigurationError
from lava_dispatcher.logical import Boot
from lava_dispatcher.actions.boot import (
    BootAction,
    AutoLoginAction,
    BootloaderCommandOverlay,
    BootloaderCommandsAction,
    BootloaderSecondaryMedia,
    OverlayUnpack,
    BootloaderInterruptAction,
)
from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
from lava_dispatcher.shell import ExpectShellSession
from lava_dispatcher.connections.lxc import ConnectLxc
from lava_dispatcher.connections.serial import ConnectDevice
from lava_dispatcher.power import ResetDevice
from lava_dispatcher.utils.strings import map_kernel_uboot
from lava_dispatcher.utils.storage import FlashUBootUMSAction
from lava_dispatcher.utils.udev import WaitDevicePathAction


class UBoot(Boot):
    """
    The UBoot method prepares the command to run on the dispatcher but this
    command needs to start a new connection and then interrupt u-boot.
    An expect shell session can then be handed over to the UBootAction.
    self.run_command is a blocking call, so Boot needs to use
    a direct spawn call via ShellCommand (which wraps pexpect.spawn) then
    hand this pexpect wrapper to subsequent actions as a shell connection.
    """

    compatibility = 1

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = UBootAction()
        self.action.section = self.action_type
        self.action.job = self.job
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        if parameters["method"] != "u-boot":
            return False, '"method" was not "u-boot"'
        if "commands" not in parameters:
            raise ConfigurationError("commands not specified in boot parameters")
        if "u-boot" in device["actions"]["boot"]["methods"]:
            return True, "accepted"
        return False, '"u-boot" was not in the device configuration boot methods'


class UBootAction(BootAction):
    """
    Wraps the Retry Action to allow for actions which precede
    the reset, e.g. Connect.
    """

    name = "uboot-action"
    description = "interactive uboot action"
    summary = "pass uboot commands"

    def validate(self):
        super().validate()
        if "type" in self.parameters:
            self.logger.warning(
                "Specifying a type in the boot action is deprecated. "
                "Please specify the kernel type in the deploy parameters."
            )

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        # customize the device configuration for this job
        self.internal_pipeline.add_action(UBootSecondaryMedia())
        self.internal_pipeline.add_action(BootloaderCommandOverlay())
        self.internal_pipeline.add_action(ConnectDevice())
        self.internal_pipeline.add_action(UBootRetry())


class UBootRetry(BootAction):

    name = "uboot-retry"
    description = "interactive uboot retry action"
    summary = "uboot commands with retry"

    def __init__(self):
        super().__init__()
        self.method_params = None
        self.usb_mass_device = None

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.method_params = self.job.device["actions"]["boot"]["methods"]["u-boot"][
            "parameters"
        ]
        self.usb_mass_device = self.method_params.get("uboot_mass_storage_device")
        # establish a new connection before trying the reset
        self.internal_pipeline.add_action(ResetDevice())
        self.internal_pipeline.add_action(BootloaderInterruptAction())
        if self.method_params.get("uboot_ums_flash", False):
            self.internal_pipeline.add_action(
                BootloaderCommandsAction(expect_final=False)
            )
            self.internal_pipeline.add_action(
                WaitDevicePathAction(self.usb_mass_device)
            )
            self.internal_pipeline.add_action(FlashUBootUMSAction(self.usb_mass_device))
            self.internal_pipeline.add_action(ResetDevice())
        else:
            self.internal_pipeline.add_action(BootloaderCommandsAction())
        if self.has_prompts(parameters):
            self.internal_pipeline.add_action(AutoLoginAction())
            if self.test_has_shell(parameters):
                self.internal_pipeline.add_action(ExpectShellSession())
                if "transfer_overlay" in parameters:
                    self.internal_pipeline.add_action(OverlayUnpack())
                self.internal_pipeline.add_action(ExportDeviceEnvironment())

    def validate(self):
        super().validate()
        self.set_namespace_data(
            action=self.name,
            label="bootloader_prompt",
            key="prompt",
            value=self.job.device["actions"]["boot"]["methods"]["u-boot"]["parameters"][
                "bootloader_prompt"
            ],
        )

    def run(self, connection, max_end_time):
        connection = super().run(connection, max_end_time)
        self.set_namespace_data(
            action="shared", label="shared", key="connection", value=connection
        )
        return connection


class UBootSecondaryMedia(BootloaderSecondaryMedia):
    """
    Idempotent action which sets the static data only used when this is a boot of secondary media
    already deployed.
    """

    name = "uboot-from-media"
    description = (
        "let uboot know where to find the kernel in the image on secondary media"
    )
    summary = "set uboot strings for deployed media"

    def validate(self):
        if "media" not in self.job.device.get("parameters", []):
            return
        media_keys = self.job.device["parameters"]["media"].keys()
        if self.parameters["commands"] not in list(media_keys):
            return
        super().validate()
        if "kernel_type" not in self.parameters:
            self.errors = "Missing kernel_type for secondary media boot"
        self.logger.debug("Mapping kernel_type: %s", self.parameters["kernel_type"])
        bootcommand = map_kernel_uboot(
            self.parameters["kernel_type"], self.job.device.get("parameters")
        )
        self.logger.debug("Using bootcommand: %s", bootcommand)
        self.set_namespace_data(
            action="uboot-prepare-kernel",
            label="kernel-type",
            key="kernel-type",
            value=self.parameters.get("kernel_type", ""),
        )
        self.set_namespace_data(
            action="uboot-prepare-kernel",
            label="bootcommand",
            key="bootcommand",
            value=bootcommand,
        )

        media_params = self.job.device["parameters"]["media"][
            self.parameters["commands"]
        ]
        if (
            self.get_namespace_data(
                action="storage-deploy", label="u-boot", key="device"
            )
            not in media_params
        ):
            self.errors = "%s does not match requested media type %s" % (
                self.get_namespace_data(
                    action="storage-deploy", label="u-boot", key="device"
                ),
                self.parameters["commands"],
            )
        if not self.valid:
            return
        self.set_namespace_data(
            action=self.name,
            label="uuid",
            key="boot_part",
            value="%s:%s"
            % (
                media_params[
                    self.get_namespace_data(
                        action="storage-deploy", label="u-boot", key="device"
                    )
                ]["device_id"],
                self.parameters["boot_part"],
            ),
        )


class UBootEnterFastbootAction(BootAction):

    name = "uboot-enter-fastboot"
    description = "interactive uboot enter fastboot action"
    summary = "uboot commands to enter fastboot mode"

    def __init__(self):
        super().__init__()
        self.params = {}

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        # establish a new connection before trying the reset
        self.internal_pipeline.add_action(ResetDevice())
        # need to look for Hit any key to stop autoboot
        self.internal_pipeline.add_action(BootloaderInterruptAction())
        self.internal_pipeline.add_action(ConnectLxc())

    def validate(self):
        super().validate()
        if "u-boot" not in self.job.device["actions"]["deploy"]["methods"]:
            self.errors = "uboot method missing"

        self.params = self.job.device["actions"]["deploy"]["methods"]["u-boot"][
            "parameters"
        ]
        if (
            "commands"
            not in self.job.device["actions"]["deploy"]["methods"]["u-boot"][
                "parameters"
            ]["fastboot"]
        ):
            self.errors = "uboot command missing"

    def run(self, connection, max_end_time):
        connection = super().run(connection, max_end_time)
        connection.prompt_str = self.params["bootloader_prompt"]
        self.logger.debug("Changing prompt to %s", connection.prompt_str)
        self.wait(connection)
        i = 1
        commands = self.job.device["actions"]["deploy"]["methods"]["u-boot"][
            "parameters"
        ]["fastboot"]["commands"]

        for line in commands:
            connection.sendline(line, delay=self.character_delay)
            if i != (len(commands)):
                self.wait(connection)
                i += 1

        return connection



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-17 09:21
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_results_app", "0014_xaxis_maxlength_increase")]

    operations = [
        migrations.AlterField(
            model_name="testcase",
            name="result",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "Test passed"),
                    (1, "Test failed"),
                    (2, "Test skipped"),
                    (3, "Unknown outcome"),
                ],
                db_index=True,
                help_text="Result classification to pass/fail group",
                verbose_name="Result",
            ),
        )
    ]



# Copyright (C) 2015 Linaro Limited
#
# Author: Stevan Radakovic <stevan.radakovic@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.action import Action


class ExportDeviceEnvironment(Action):
    """
    Exports environment variables found in common data on to the device.
    """

    name = "export-device-env"
    description = "Exports environment variables to the device"
    summary = "Exports environment variables action"

    def __init__(self):
        super().__init__()
        self.env = []

    def validate(self):
        super().validate()
        shell_file = self.get_namespace_data(
            action="deploy-device-env", label="environment", key="shell_file"
        )
        environment = self.get_namespace_data(
            action="deploy-device-env", label="environment", key="env_dict"
        )
        if not environment:
            return
        # Append export commands to the shell init file.
        # Retain quotes into the final shell.
        for key in environment:
            self.env.append(
                "echo export %s=\\'%s\\' >> %s" % (key, environment[key], shell_file)
            )

    def run(self, connection, max_end_time):

        if not connection:
            return

        connection = super().run(connection, max_end_time)

        shell_file = self.get_namespace_data(
            action="deploy-device-env", label="environment", key="shell_file"
        )

        for line in self.env:
            connection.sendline(line, delay=self.character_delay)

        if shell_file:
            connection.sendline(". %s" % shell_file, delay=self.character_delay)

        return connection



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('dashboard_app', '0004_imagereportchart_is_delta'),
    ]

    operations = [
        migrations.AddField(
            model_name='imagereportchart',
            name='chart_height',
            field=models.PositiveIntegerField(default=200, verbose_name='Chart height', validators=[django.core.validators.MinValueValidator(200), django.core.validators.MaxValueValidator(400)]),
            preserve_default=True,
        ),
    ]



# Copyright (C) 2016, 2017 Collabora Ltd.
#
# Author: Tomeu Vizoso <tomeu.vizoso@collabora.com>
# Author: Guillaume Tucker <guillaume.tucker@collabora.com>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.


import os
import unittest
from lava_dispatcher.device import NewDevice
from lava_dispatcher.parser import JobParser
from lava_dispatcher.tests.test_basic import StdoutTestCase
from lava_dispatcher.tests.utils import DummyLogger, infrastructure_error


class DepthchargeFactory:
    """
    Not Model based, this is not a Django factory.
    Factory objects are dispatcher based classes, independent
    of any database objects.
    """

    def create_jaq_job(self, filename):  # pylint: disable=no-self-use
        device = NewDevice(
            os.path.join(os.path.dirname(__file__), "../devices/jaq-01.yaml")
        )
        yaml = os.path.join(os.path.dirname(__file__), filename)
        with open(yaml) as sample_job_data:
            parser = JobParser()
            job = parser.parse(sample_job_data, device, 4212, None, "")
            job.logger = DummyLogger()
        return job


class TestDepthchargeAction(StdoutTestCase):
    def setUp(self):
        super().setUp()
        self.factory = DepthchargeFactory()

    @unittest.skipIf(infrastructure_error("mkimage"), "mkimage not installed")
    def test_depthcharge(self):
        job = self.factory.create_jaq_job("sample_jobs/depthcharge.yaml")
        self.assertIsNotNone(job)

        description_ref = self.pipeline_reference("depthcharge.yaml", job=job)
        self.assertEqual(description_ref, job.pipeline.describe(False))

        self.assertIsNone(job.validate())

        self.assertEqual(
            [action.name for action in job.pipeline.actions],
            ["tftp-deploy", "depthcharge-action", "finalize"],
        )

        tftp = [
            action for action in job.pipeline.actions if action.name == "tftp-deploy"
        ][0]
        prep_overlay = [
            action
            for action in tftp.pipeline.actions
            if action.name == "prepare-tftp-overlay"
        ][0]
        prep_kernel = [
            action
            for action in prep_overlay.pipeline.actions
            if action.name == "prepare-kernel"
        ][0]
        self.assertEqual(
            [action.name for action in prep_kernel.internal_pipeline.actions],
            ["prepare-fit"],
        )

        prep_fit = [
            action
            for action in prep_kernel.pipeline.actions
            if action.name == "prepare-fit"
        ][0]
        params = {
            "arch": "neo-gothic",
            "load_addr": "0x1234",
            "kernel": "/some/zImage",
            "dtb": "/some/file.dtb",
            "ramdisk": "/some/ramdisk.cpio",
            "fit_path": "/does/not/exist",
        }
        cmd_ref = 'mkimage \
-D "-I dts -O dtb -p 2048" \
-f auto \
-A {arch} \
-O linux \
-T kernel \
-C None \
-d {kernel} \
-a {load_addr} \
-b {dtb} \
-i {ramdisk} \
{fit_path}'.format(
            **params
        )
        cmd = prep_fit._make_mkimage_command(params)  # pylint: disable=protected-access
        self.assertEqual(cmd_ref, " ".join(cmd))

        depthcharge = [
            action
            for action in job.pipeline.actions
            if action.name == "depthcharge-action"
        ][0]
        self.assertEqual(
            [action.name for action in depthcharge.internal_pipeline.actions],
            ["depthcharge-overlay", "connect-device", "depthcharge-retry"],
        )

        retry = [
            action
            for action in depthcharge.internal_pipeline.actions
            if action.name == "depthcharge-retry"
        ][0]
        self.assertEqual(
            [action.name for action in retry.internal_pipeline.actions],
            ["reset-device", "depthcharge-start", "bootloader-commands"],
        )



# Copyright (C) 2017 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.action import Action
from lava_common.exceptions import ConfigurationError


class CommandAction(Action):

    compatibility = 1
    name = "user-command"
    description = "execute one of the commands listed by the admin"
    summary = "execute commands"

    def __init__(self):
        super().__init__()
        self.section = "command"
        self.cmd = None
        self.ran = False

    def validate(self):
        super().validate()
        cmd_name = self.parameters["name"]
        try:
            user_commands = self.job.device["commands"]["users"]
        except KeyError:
            raise ConfigurationError("Unable to get device.commands.users dictionary")

        try:
            self.cmd = user_commands[cmd_name]
            if not isinstance(self.cmd["do"], str) or not isinstance(
                self.cmd.get("undo", ""), str
            ):
                raise ConfigurationError(
                    'User command "%s" is invalid: '
                    "'do' and 'undo' should be strings" % cmd_name
                )
            return True
        except KeyError:
            self.errors = "Unknown user command '%s'" % cmd_name
            return False

    def run(self, connection, max_end_time):
        connection = super().run(connection, max_end_time)

        self.logger.debug("Running user command '%s'", self.parameters["name"])
        self.ran = True
        self.run_command(self.cmd["do"].split(" "), allow_silent=True)
        return connection

    def cleanup(self, connection):
        if not self.ran:
            self.logger.debug("Skipping %s 'undo' as 'do' was not called", self.name)
            return

        if self.cmd is not None and "undo" in self.cmd:
            self.logger.debug(
                "Running cleanup for user command '%s'", self.parameters["name"]
            )
            if not isinstance(self.cmd["undo"], str):
                self.logger.error("Unable to run cleanup: 'undo' is not a string")
            else:
                self.run_command(self.cmd["undo"].split(" "), allow_silent=True)



# Copyright (C) 2017 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.tests.test_basic import Factory, StdoutTestCase


class TestCommand(StdoutTestCase):
    def setUp(self):
        super().setUp()
        factory = Factory()
        self.job = factory.create_kvm_job("sample_jobs/kvm-command.yaml")

    def test_pipeline(self):
        description_ref = self.pipeline_reference("kvm-command.yaml")
        self.assertEqual(description_ref, self.job.pipeline.describe(False))

        command = [
            action
            for action in self.job.pipeline.actions
            if action.name == "user-command"
        ][0]
        self.assertEqual(command.parameters["name"], "user_command_to_run")
        self.assertEqual(command.timeout.duration, 60)



#! /usr/bin/python3

"""
This script is particularly intended for those adding new devices to LAVA
and developing new jinja templates to create the per-device configuration
in the Device Dictionary database table.

In a production system, the devices directory will be stored in
/etc/lava-server/dispatcher-config/devices/<hostname>.jinja2

When developers are working on new support directly from the command line
lava-dispatch or developing new templates, this script can be used to match the
template output with existing templates.

The path used needs to contain both the jinja2 device-type template in a
device-types/ directory *and* the jinja2 device dictionary file for the device
to review in a devices/ directory.

(This script will go into the lava-dev binary package.)

"""

#  Copyright 2014 Linaro Limited
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

# pylint gets confused: commands have no shebang, but the file is not a module.
# pylint: disable=invalid-name

import os
import yaml
import argparse
from jinja2 import Environment, FileSystemLoader

# pylint: disable=superfluous-parens,maybe-no-member


def main():

    parser = argparse.ArgumentParser(description="LAVA Dispatcher template helper")
    parser.add_argument(
        "--device", type=str, required=True, help="Path to the device template file"
    )
    parser.add_argument(
        "--path",
        default="/etc/lava-server/dispatcher-config/",
        type=str,
        help="Path to the device-types template folder",
    )
    args = parser.parse_args()

    env = Environment(  # nosec rendering to YAML
        loader=FileSystemLoader(
            [
                os.path.join(args.path, "devices"),
                os.path.join(args.path, "device-types"),
            ]
        ),
        trim_blocks=True,
        autoescape=False,
    )
    if not os.path.exists(
        os.path.join(args.path, "devices", "%s.jinja2" % args.device)
    ):
        print("Cannot find %s device configuration file" % args.device)
        return
    template = env.get_template("%s.jinja2" % args.device)
    ctx = {}
    config = template.render(**ctx)

    print("YAML config")
    print("===========")
    print(config)
    print("Parsed config")
    print("=============")
    print(yaml.safe_load(config))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-06 15:22
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_results_app", "0013_buglinks")]

    operations = [
        migrations.AlterField(
            model_name="chartquery",
            name="xaxis_attribute",
            field=models.CharField(
                blank=True, max_length=100, null=True, verbose_name="X-axis attribute"
            ),
        )
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2011-2018 Linaro Limited
#
# Author: Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.

import pathlib
import struct

PACK_FORMAT = "=Q"
PACK_SIZE = struct.calcsize(PACK_FORMAT)


def _build_index(directory):
    with open(str(directory / "output.yaml"), "rb") as f_log:
        with open(str(directory / "output.idx"), "wb") as f_idx:
            f_idx.write(struct.pack(PACK_FORMAT, 0))
            line = f_log.readline()
            while line:
                f_idx.write(struct.pack(PACK_FORMAT, f_log.tell()))
                line = f_log.readline()


def _get_line_offset(f_idx, line):
    f_idx.seek(PACK_SIZE * line, 0)
    data = f_idx.read(PACK_SIZE)
    if data:
        return struct.unpack(PACK_FORMAT, data)[0]
    else:
        return None


def line_count(f_idx):
    return int(f_idx.tell() / PACK_SIZE)


def read_logs(dir_name, start=0, end=None):
    directory = pathlib.Path(dir_name)
    if not (directory / "output.idx").exists():
        _build_index(directory)

    with open(str(directory / "output.idx"), "rb") as f_idx:
        start_offset = _get_line_offset(f_idx, start)
        if start_offset is None:
            return ""
        with open(str(directory / "output.yaml"), "rb") as f_log:
            f_log.seek(start_offset)
            if end is None:
                return f_log.read().decode("utf-8")
            end_offset = _get_line_offset(f_idx, end)
            if end_offset is None:
                return f_log.read().decode("utf-8")
            if end_offset <= start_offset:
                return ""
            return f_log.read(end_offset - start_offset).decode("utf-8")


def write_logs(f_log, f_idx, line):
    f_idx.write(struct.pack(PACK_FORMAT, f_log.tell()))
    f_idx.flush()
    f_log.write(line)
    f_log.flush()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  javascript.py
#
#  Copyright 2015 Neil Williams <codehelp@debian.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import yaml
import argparse
import subprocess  # nosec - internal

# pylint: disable=too-many-branches

# Adds a dependency on python-yaml when used during a build.


def check_os(os_name, data):
    """
    Isolate just the data for the requested OS
    Only debian supported so far.
    """
    for key, list_data in data.items():
        if key == os_name:
            yield list_data


def handle_embedded(os_name, data, dirname, simulate=False):
    """
    Remove the packaged duplicate
    Create a symlink from the external package to the old name
    $(RM) $(CURDIR)/$(JS_DIR)/lava_server/static/lava_server/js/jquery-1.11.0.min.js
    """
    python_dir = None
    dependencies = {}
    os_check = check_os(os_name, data)
    for os_data in next(os_check):
        for dkey, value in os_data.items():
            if dkey == "python_dir":
                python_dir = value
                if python_dir.startswith("/"):
                    python_dir = str(python_dir[1:])
            elif dkey == "package":
                package = os_data
                print(
                    'Linking files from "%s" into "%s"'
                    % (package["package"], package["lava_directory"])
                )
                if "version" in package:
                    dependencies[package["package"]] = "(>= %s)" % package["version"]
                else:
                    dependencies[package["package"]] = None
            elif dkey == "replacements":
                package = os_data
                for ours, external in package["replacements"].items():
                    ext_path = os.path.join(package["directory"], external)
                    our_path = os.path.join(
                        dirname, python_dir, package["lava_directory"], ours
                    )
                    if not os.path.exists(ext_path):
                        print("skipping symlink for %s" % ext_path)
                        continue
                    if not simulate:
                        if not os.path.exists(our_path):
                            raise RuntimeError("missing %s" % our_path)
                        os.unlink(our_path)
                        os.symlink(ext_path, our_path)
                    else:
                        print("rm %s" % our_path)
                        print("ln -s %s %s" % (ext_path, our_path))
    return dependencies


def uglify(os_name, data, dirname, remove=False, simulate=False):
    """
    """
    python_dir = None
    os_check = check_os(os_name, data)
    for os_data in next(os_check):
        for dkey, value in os_data.items():
            if dkey == "python_dir":
                python_dir = value
                if python_dir.startswith("/"):
                    python_dir = str(python_dir[1:])
            elif dkey == "uglify":
                package = os_data
                lava_dir = package["lava_directory"]
                dest_dir = package["destination"]

                for file_name, dest_name in package["files"].items():

                    orig_path = os.path.join(lava_dir, file_name)
                    install_path = os.path.join(
                        dirname, python_dir, lava_dir, file_name
                    )
                    dest_path = os.path.join(dirname, python_dir, dest_dir, dest_name)

                    if not simulate:
                        try:
                            subprocess.check_call(  # nosec - internal
                                [
                                    "/usr/bin/uglifyjs",
                                    orig_path,
                                    "-o",
                                    dest_path,
                                    "-c",
                                    "-m",
                                ],
                                stderr=open(os.devnull, "wb"),
                            )
                        except Exception as e:
                            print(e)

                        if remove:
                            if not os.path.exists(install_path):
                                print(
                                    "WARNING: JS file %s does not exist"
                                    % (install_path)
                                )
                                continue
                            os.unlink(install_path)
                    else:
                        print("uglifyjs %s -o %s -c -m" % (orig_path, dest_path))

    return None


def main():
    """
    Parse options and load the supporting YAML file.
    Where debian is used, debian === debian-based
    """
    parser = argparse.ArgumentParser(description="Handle embedded javascript")
    parser.add_argument(
        "-f",
        "--filename",
        required=True,
        help="YAML file describing embedded javascript",
    )
    parser.add_argument(
        "-r", "--remove", action="store_true", help="Remove original js files from .deb"
    )
    parser.add_argument(
        "-s", "--simulate", action="store_true", help="Only echo the commands"
    )

    args = parser.parse_args()
    data = yaml.safe_load(open(args.filename, "r"))
    # only have data for debian-based packages so far.
    dependencies = handle_embedded("debian", data, os.getcwd(), args.simulate)
    dep_list = []
    for package, constraint in dependencies.items():
        if constraint:
            dep_list.append("%s %s" % (package, constraint))
        else:
            dep_list.append(package)
    if args.simulate:
        # only useful for Debian-based
        print("")
        print("Build-Depends:", ", ".join(sorted(dep_list)))
        print("Depends:", ", ".join(sorted(dep_list)))

    uglify("debian", data, os.getcwd(), args.remove, args.simulate)
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Linaro Limited
#
# Author: Senthil Kumaran S <senthil.kumaran@linaro.org>
#         Remi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses>.

# Adds the device matching the device-node
# to the lxc identified by lxc-name
#
# Usage: lava_lxc_device_add.py <lxc-name> <device-node>

import os
import argparse
import logging
import subprocess  # nosec - internal
import sys
import time
import syslog

from lava_dispatcher.log import YAMLLogger

# Wait 10s maximum to close the socket
LINGER = 10000

# pylint: disable=line-too-long,missing-docstring


def setup_logger(options):
    # Pipeline always log as YAML so change the base logger.
    # Every calls to logging.getLogger will now return a YAMLLogger
    logging.setLoggerClass(YAMLLogger)

    # The logger can be used by the parser and the Job object in all phases.
    logger = logging.getLogger("dispatcher")
    if options.logging_url is not None:
        if options.master_cert and options.slave_cert:
            if not os.path.exists(options.master_cert) or not os.path.exists(
                options.slave_cert
            ):
                syslog.syslog(
                    "[%s] Unable to find certificates for %s"
                    % (options.job_id, options.logging_url)
                )
                return None
        # pylint: disable=no-member
        handler = logger.addZMQHandler(
            options.logging_url,
            options.master_cert,
            options.slave_cert,
            options.job_id,
            options.ipv6,
        )
    else:
        syslog.syslog("[%s] Logging to streamhandler" % options.job_id)
        logger.addHandler(logging.StreamHandler())

    return logger


def main():
    # Configure the parser
    parser = argparse.ArgumentParser()

    parser.add_argument("--lxc-name", required=True, help="Name of the lxc container")
    parser.add_argument("--device-node", required=True, help="Path to the device node")

    group = parser.add_argument_group("logging")
    group.add_argument("--job-id", required=True, metavar="ID", help="Job identifier.")
    group.add_argument(
        "--logging-url",
        metavar="URL",
        default=None,
        help="URL of the ZMQ socket to send the logs to the master",
    )
    group.add_argument(
        "--master-cert", default=None, metavar="PATH", help="Master certificate file"
    )
    group.add_argument(
        "--slave-cert", default=None, metavar="PATH", help="Slave certificate file"
    )
    group.add_argument("--ipv6", action="store_true", default=False, help="Enable IPv6")

    # Parse the command line
    options = parser.parse_args()
    lxc_name = options.lxc_name

    # Setup the logger
    logger = setup_logger(options)
    if not logger:
        syslog.syslog("[%s] Unable to setup the logger, exiting." % (options.job_id))
        return 1

    start = time.gmtime()
    uniq_str = "udev_trigger-%s-%02d:%02d:%02d" % (
        lxc_name,
        start.tm_hour,
        start.tm_min,
        start.tm_sec,
    )

    device = "/dev/%s" % options.device_node

    if not os.path.exists(device):
        logger.debug("Skipping node not in /dev/ : %s", options.device_node)
        syslog.syslog(
            "[%s] Skipping node not in /dev/ : %s"
            % (options.job_id, options.device_node)
        )
        return 0

    lxc_cmd = ["lxc-device", "-n", lxc_name, "add", device]
    try:
        output = subprocess.check_output(  # nosec - internal
            lxc_cmd, stderr=subprocess.STDOUT
        )
        output = output.decode("utf-8", errors="replace")
        logger.debug(output)
        logger.info("[%s] device %s added", uniq_str, device)
        syslog.syslog("[%s] device %s added" % (uniq_str, device))
    except subprocess.CalledProcessError as exc:
        msg = "[%s] failed to add device %s: '%s'" % (uniq_str, device, exc)
        logger.error(msg)
        syslog.syslog(msg)
        syslog.syslog("[%s] Tried: %s" % (options.job_id, " ".join(lxc_cmd)))
        logger.close(linger=LINGER)  # pylint: disable=no-member
        return 2
    except Exception as exc:
        syslog.syslog("[%s] uncaught exception: %s" % (options.job_id, exc))
        logger.close(linger=LINGER)  # pylint: disable=no-member
        return 3

    logger.close(linger=LINGER)  # pylint: disable=no-member
    return 0


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-03 09:47
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0024_do_not_cascade_deletions")]

    operations = [
        migrations.AlterField(
            model_name="notification",
            name="job_status_trigger",
            field=models.CharField(
                choices=[
                    (0, "Submitted"),
                    (1, "Running"),
                    (2, "Complete"),
                    (3, "Incomplete"),
                    (4, "Canceled"),
                    (5, "Canceling"),
                ],
                default=2,
                max_length=30,
                verbose_name="Job status trigger",
            ),
        )
    ]



# Copyright (C) 2014 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

# List just the subclasses supported for this base strategy
# imported by the parser to populate the list of subclasses.

import os
from lava_dispatcher.action import Action, Pipeline, JobError, Timeout
from lava_dispatcher.logical import Deployment
from lava_dispatcher.actions.deploy.download import DownloaderAction
from lava_dispatcher.actions.deploy.overlay import OverlayAction
from lava_dispatcher.actions.deploy.apply_overlay import ApplyOverlayImage
from lava_dispatcher.actions.deploy import DeployAction
from lava_dispatcher.actions.deploy.environment import DeployDeviceEnvironment
from lava_dispatcher.utils.network import dispatcher_ip
from lava_dispatcher.utils.strings import substitute
from lava_common.constants import DD_PROMPTS


class Removable(Deployment):
    """
    Deploys an image to a usb or sata mass storage device
    *Destroys* anything on that device, including partition table
    Requires a preceding boot (e.g. ramdisk) which may have a test shell of it's own.
    Does not require the ramdisk to be able to mount the usb storage, just for the kernel
    to be able to see the device (the filesystem will be replaced anyway).

    SD card partitions will use a similar approach but the UUID will be fixed in the device
    configuration and specifying a restricted UUID will invalidate the job to protect the bootloader.

    """

    compatibility = 1
    name = "removeable"

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = MassStorage()
        self.action.job = self.job
        self.action.section = self.action_type
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        media = parameters.get("to")
        job_device = parameters.get("device")

        # Is the media supported?
        if media not in ["sata", "sd", "usb"]:
            return False, '"media" was not "sata", "sd", or "usb"'
        # "parameters.media" is not defined for every devices
        if "parameters" not in device or "media" not in device["parameters"]:
            return (
                False,
                '"parameters" was not in the device or "media" was not in the parameters',
            )
        # Is the device allowing this method?
        if job_device not in device["parameters"]["media"].get(media, {}):
            return False, 'media was not in the device "media" parameters'
        # Is the configuration correct?
        if "uuid" in device["parameters"]["media"][media].get(job_device, {}):
            return True, "accepted"
        return (
            False,
            '"uuid" was not in the parameters for the media device %s' % job_device,
        )


class DDAction(Action):
    """
    Runs dd or a configurable writer against the realpath of the symlink
    provided by the static device information: device['parameters']['media']
    (e.g. usb-SanDisk_Ultra_20060775320F43006019-0:0) in /dev/disk/by-id/ of
    the initial deployment, on device.
    """

    name = "dd-image"
    description = "deploy image to drive"
    summary = "write image to drive"

    def __init__(self):
        super().__init__()
        self.timeout = Timeout(self.name, 600)
        self.boot_params = None
        self.tool_prompts = None
        self.tool_flags = None

    def validate(self):
        super().validate()
        if "device" not in self.parameters:
            self.errors = "missing device for deployment"

        download_params = self.parameters.get("download")
        writer_params = self.parameters.get("writer")
        if not download_params and not writer_params:
            self.errors = "Neither a download nor a write tool found in parameters"

        if download_params:
            if "tool" not in download_params:
                self.errors = "missing download or writer tool for deployment"
            if "options" not in download_params:
                self.errors = "missing options for download tool"
            if "prompt" not in download_params:
                self.errors = "missing prompt for download tool"
            if not os.path.isabs(download_params["tool"]):
                self.errors = "download tool parameter needs to be an absolute path"

        if writer_params:
            if "tool" not in writer_params:
                self.errors = "missing writer tool for deployment"
            if "options" not in writer_params:
                self.errors = "missing options for writer tool"
            if "download" not in self.parameters:
                if "prompt" not in writer_params:
                    self.errors = "missing prompt for writer tool"
            if not os.path.isabs(writer_params["tool"]):
                self.errors = "writer tool parameter needs to be an absolute path"

        if self.parameters["to"] not in self.job.device["parameters"].get("media", {}):
            self.errors = (
                "media '%s' unavailable for this device" % self.parameters["to"]
            )

        # The `image' parameter can be either directly in the Action parameters
        # if there is a single image file, or within `images' if there are
        # multiple image files.  In either case, there needs to be one `image'
        # parameter.
        img_params = self.parameters.get("images", self.parameters)
        if "image" not in img_params:
            self.errors = "Missing image parameter"

        # No need to go further if an error was already detected
        if not self.valid:
            return

        tool_params = self.parameters.get("tool")
        if tool_params:
            self.tool_prompts = tool_params.get("prompts", DD_PROMPTS)
            self.tool_flags = tool_params.get("flags")
        else:
            self.tool_prompts = DD_PROMPTS

        if not isinstance(self.tool_prompts, list):
            self.errors = "'tool prompts' should be a list"
        else:
            for msg in self.tool_prompts:
                if not msg:
                    self.errors = "items of 'tool prompts' cannot be empty"

        uuid_required = False
        self.boot_params = self.job.device["parameters"]["media"][self.parameters["to"]]
        uuid_required = self.boot_params.get("UUID-required", False)

        if uuid_required:  # FIXME unit test required
            if "uuid" not in self.boot_params[self.parameters["device"]]:
                self.errors = "A UUID is required for %s" % (self.parameters["device"])
            if "root_part" in self.boot_params[self.parameters["device"]]:
                self.errors = "'root_part' is not valid as a UUID is required"
        if self.parameters["device"] in self.boot_params:
            self.set_namespace_data(
                action=self.name,
                label="u-boot",
                key="boot_part",
                value=self.boot_params[self.parameters["device"]]["device_id"],
            )

    def run(self, connection, max_end_time):  # pylint: disable=too-many-locals
        """
        Retrieve the decompressed image from the dispatcher by calling the tool specified
        by the test writer, from within the test image of the first deployment, using the
        device to write directly to the secondary media, without needing to cache on the device.
        """
        connection = super().run(connection, max_end_time)
        d_file = self.get_namespace_data(
            action="download-action", label="image", key="file"
        )
        if not d_file:
            self.logger.debug("Skipping %s - nothing downloaded")
            return connection
        decompressed_image = os.path.basename(d_file)
        try:
            device_path = os.path.realpath(
                "/dev/disk/by-id/%s"
                % self.boot_params[self.parameters["device"]]["uuid"]
            )
        except OSError:
            raise JobError(
                "Unable to find disk by id %s"
                % self.boot_params[self.parameters["device"]]["uuid"]
            )
        storage_suffix = self.get_namespace_data(
            action="storage-deploy", label="storage", key="suffix"
        )
        if not storage_suffix:
            storage_suffix = ""
        suffix = "%s/%s" % ("tmp", storage_suffix)

        # As the test writer can use any tool we cannot predict where the
        # download URL will be positioned in the download command.
        # Providing the download URL as a substitution option gets round this
        ip_addr = dispatcher_ip(self.job.parameters["dispatcher"])
        download_url = "http://%s/%s/%s" % (ip_addr, suffix, decompressed_image)
        substitutions = {"{DOWNLOAD_URL}": download_url, "{DEVICE}": device_path}

        download_cmd = None
        download_params = self.parameters.get("download")
        if download_params:
            download_options = substitute([download_params["options"]], substitutions)[
                0
            ]
            download_cmd = " ".join([download_params["tool"], download_options])

        writer_params = self.parameters.get("writer")
        if writer_params:
            tool_options = substitute([writer_params["options"]], substitutions)[0]
            tool_cmd = [writer_params["tool"], tool_options]
        else:
            tool_cmd = [
                "dd of='{}' bs=4M".format(device_path)
            ]  # busybox dd does not support other flags
        if self.tool_flags:
            tool_cmd.append(self.tool_flags)
        cmd = " ".join(tool_cmd)

        cmd_line = " ".join([download_cmd, "|", cmd]) if download_cmd else cmd

        # set prompt to either `download' or `writer' prompt to ensure that the
        # secondary deployment has started
        prompt_string = connection.prompt_str
        prompt_param = download_params or writer_params
        connection.prompt_str = prompt_param["prompt"]
        self.logger.debug("Changing prompt to %s", connection.prompt_str)

        connection.sendline(cmd_line)
        self.wait(connection)
        if not self.valid:
            self.logger.error(self.errors)

        # change prompt string to list of dd outputs
        connection.prompt_str = self.tool_prompts
        self.logger.debug("Changing prompt to %s", connection.prompt_str)
        self.wait(connection)

        # set prompt back once secondary deployment is complete
        connection.prompt_str = prompt_string
        self.logger.debug("Changing prompt to %s", connection.prompt_str)
        self.set_namespace_data(
            action="shared", label="shared", key="connection", value=connection
        )
        return connection


class MassStorage(DeployAction):  # pylint: disable=too-many-instance-attributes

    name = "storage-deploy"
    description = "Deploy image to mass storage"
    summary = "write image to storage"

    def __init__(self):
        super().__init__()
        self.suffix = None
        self.image_path = None

    def validate(self):
        super().validate()
        # if 'image' not in self.parameters.keys():
        #     self.errors = "%s needs an image to deploy" % self.name
        if "device" not in self.parameters:
            self.errors = "No device specified for mass storage deployment"
        if not self.valid:
            return

        self.set_namespace_data(
            action=self.name,
            label="u-boot",
            key="device",
            value=self.parameters["device"],
        )
        suffix = os.path.join(*self.image_path.split("/")[-2:])
        self.set_namespace_data(
            action=self.name, label="storage", key="suffix", value=suffix
        )

    def populate(self, parameters):
        """
        The dispatcher does the first download as the first deployment is not guaranteed to
        have DNS resolution fully working, so we can use the IP address of the dispatcher
        to get it (with the advantage that the dispatcher decompresses it so that the ramdisk
        can pipe the raw image directly from wget to dd.
        This also allows the use of local file:// locations which are visible to the dispatcher
        but not the device.
        """
        self.image_path = self.mkdtemp()
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        if self.test_needs_overlay(parameters):
            self.internal_pipeline.add_action(
                OverlayAction()
            )  # idempotent, includes testdef
        uniquify = parameters.get("uniquify", True)
        if "images" in parameters:
            for k in sorted(parameters["images"].keys()):
                self.internal_pipeline.add_action(
                    DownloaderAction(k, path=self.image_path, uniquify=uniquify)
                )
                if parameters["images"][k].get("apply-overlay", False):
                    if self.test_needs_overlay(parameters):
                        self.internal_pipeline.add_action(ApplyOverlayImage())
            self.internal_pipeline.add_action(DDAction())
        elif "image" in parameters:
            self.internal_pipeline.add_action(
                DownloaderAction("image", path=self.image_path, uniquify=uniquify)
            )
            if self.test_needs_overlay(parameters):
                self.internal_pipeline.add_action(ApplyOverlayImage())
            self.internal_pipeline.add_action(DDAction())

        # FIXME: could support tarballs too
        if self.test_needs_deployment(parameters):
            self.internal_pipeline.add_action(DeployDeviceEnvironment())



# coding: utf-8
#
# Copyright (C) 2018 Linaro Limited
#
# Author: Rémi Duraffort <remi.duraffort@linaro.org>
#
# This file is part of LAVA.
#
# LAVA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from voluptuous import Any, Match, Optional, Required

from lava_common.schemas import test


def schema(live=False):
    base = {
        Required("monitors"): [
            {
                Required("name"): str,
                Required("start"): str,
                Required("end"): str,
                Required("pattern"): str,
                Optional("fixupdict"): {str: Any("pass", "fail", "skip", "unknown")},
            }
        ]
    }
    return {**test.schema(live), **base}



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-02 04:04
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0012_auto_20160208_1600")]

    operations = [
        migrations.AlterField(
            model_name="devicetype",
            name="health_denominator",
            field=models.IntegerField(
                choices=[(0, "hours"), (1, "jobs")],
                default=0,
                help_text="Choose to submit a health check every N hours or every N jobs. Balance against the duration of a health check job and the average job duration.",
                verbose_name="Initiate health checks by hours or by jobs.",
            ),
        )
    ]



# Copyright (C) 2017 Linaro Limited
#
# Author: Dean Birch <dean.birch@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

from lava_dispatcher.action import Pipeline
from lava_dispatcher.actions.boot import (
    AutoLoginAction,
    BootloaderCommandOverlay,
    OverlayUnpack,
    BootloaderCommandsAction,
    BootAction,
)

from lava_dispatcher.actions.boot.environment import ExportDeviceEnvironment
from lava_dispatcher.actions.boot.uefi_menu import UEFIMenuInterrupt, UefiMenuSelector
from lava_dispatcher.logical import Boot
from lava_dispatcher.menus.menus import MenuInterrupt, MenuConnect
from lava_dispatcher.power import ResetDevice
from lava_dispatcher.shell import ExpectShellSession
from lava_common.constants import UEFI_LINE_SEPARATOR


class UefiShell(Boot):

    compatibility = 3

    def __init__(self, parent, parameters):
        super().__init__(parent)
        self.action = UefiShellAction()
        self.action.section = self.action_type
        self.action.job = self.job
        parent.add_action(self.action, parameters)

    @classmethod
    def accepts(cls, device, parameters):
        if parameters["method"] != "uefi":
            return False, '"method" was not "uefi"'
        if "uefi" in device["actions"]["boot"]["methods"]:
            params = device["actions"]["boot"]["methods"]["uefi"]["parameters"]
            if not params:
                return (
                    False,
                    'there were no parameters in the "uefi" device configuration boot method',
                )
            if "shell_interrupt_string" not in params:
                return (
                    False,
                    '"shell_interrupt_string" was not in the uefi device configuration boot method parameters',
                )
            if "shell_interrupt_prompt" in params and "bootloader_prompt" in params:
                return True, "accepted"
        return (
            False,
            "missing or invalid parameters in the uefi device configuration boot methods",
        )


class UefiShellAction(BootAction):

    name = "uefi-shell-main-action"
    description = "UEFI shell boot action"
    summary = "run UEFI shell to system"

    def __init__(self):
        super().__init__()
        self.shell_menu = []

    def _skip_menu(self, parameters):
        # shell_menu can be set to '' to indicate there is no menu.
        if "shell_menu" in parameters:
            self.shell_menu = parameters["shell_menu"]
        elif (
            "shell_menu"
            in self.job.device["actions"]["boot"]["methods"]["uefi"]["parameters"]
        ):
            self.shell_menu = self.job.device["actions"]["boot"]["methods"]["uefi"][
                "parameters"
            ]["shell_menu"]

        if self.shell_menu and isinstance(self.shell_menu, str):
            return False
        return True

    def populate(self, parameters):
        self.internal_pipeline = Pipeline(
            parent=self, job=self.job, parameters=parameters
        )
        self.internal_pipeline.add_action(BootloaderCommandOverlay())
        self.internal_pipeline.add_action(MenuConnect())
        self.internal_pipeline.add_action(ResetDevice())
        # Newer firmware often needs no menu interaction, just press to drop to shell
        if not self._skip_menu(parameters):
            # Some older firmware, UEFI Shell has to be selected from a menu.
            self.internal_pipeline.add_action(UefiShellMenuInterrupt())
            self.internal_pipeline.add_action(UefiShellMenuSelector())
        self.internal_pipeline.add_action(UefiShellInterrupt())
        self.internal_pipeline.add_action(UefiBootloaderCommandsAction())
        if self.has_prompts(parameters):
            self.internal_pipeline.add_action(AutoLoginAction())
            if self.test_has_shell(parameters):
                self.internal_pipeline.add_action(ExpectShellSession())
                if "transfer_overlay" in parameters:
                    self.internal_pipeline.add_action(OverlayUnpack())
                self.internal_pipeline.add_action(ExportDeviceEnvironment())

    def run(self, connection, max_end_time):
        connection = super().run(connection, max_end_time)
        connection.raw_connection.linesep = UEFI_LINE_SEPARATOR
        self.set_namespace_data(
            action="shared", label="shared", key="connection", value=connection
        )
        return connection

    def validate(self):
        super().validate()
        params = self.job.device["actions"]["boot"]["methods"]["uefi"]["parameters"]
        self.set_namespace_data(
            action=self.name,
            label="bootloader_prompt",
            key="prompt",
            value=params["bootloader_prompt"],
        )


class UefiShellMenuInterrupt(UEFIMenuInterrupt):

    name = "uefi-shell-menu-interrupt"
    description = "interrupt default boot and to menu"
    summary = "interrupt default boot and to menu"

    def __init__(self):
        super().__init__()
        # Take parameters from the uefi method, not uefi menu.
        self.method = "uefi"


class UefiBootloaderCommandsAction(BootloaderCommandsAction):
    """
    Same as BootloaderCommandsAction, but uses UEFI_LINE_SEPARATOR.
    """

    def line_separator(self):
        return UEFI_LINE_SEPARATOR


class UefiShellInterrupt(MenuInterrupt):
    """
    Support for interrupting the UEFI menu and dropping to the shell.
    """

    name = "uefi-shell-interrupt"
    description = "interrupt uefi menu to get to a shell"
    summary = "first uefi interrupt"

    def run(self, connection, max_end_time):
        if not connection:
            self.logger.debug("%s called without active connection", self.name)
            return
        connection = super().run(connection, max_end_time)
        # param keys already checked in accepts() classmethod
        params = self.job.device["actions"]["boot"]["methods"]["uefi"]["parameters"]
        connection.prompt_str = params["shell_interrupt_prompt"]
        self.wait(connection)
        connection.raw_connection.send(params["shell_interrupt_string"])
        # now move on to bootloader prompt match
        return connection


class UefiShellMenuSelector(UefiMenuSelector):
    """
    Special version of the UefiMenuSelector configured to drop to the shell
    """

    name = "uefi-shell-menu-selector"
    description = "select uefi menu items to drop to a uefi shell"
    summary = "use uefi menu to drop to shell"

    def __init__(self):
        super().__init__()
        # Take parameters from the uefi method, not uefi menu.
        self.method_name = "uefi"
        # Default menu command name: drop to shell
        self.commands = "shell"

    def validate(self):
        params = self.job.device["actions"]["boot"]["methods"][self.method_name][
            "parameters"
        ]
        if "shell_menu" in self.parameters:
            self.commands = self.parameters["shell_menu"]
        elif "shell_menu" in params:
            self.commands = params["shell_menu"]

        if (
            self.commands
            in self.job.device["actions"]["boot"]["methods"][self.method_name]
        ):
            self.items = self.job.device["actions"]["boot"]["methods"][
                self.method_name
            ][self.commands]
        else:
            self.errors = "Missing menu commands for %s" % self.commands
        if "menu_boot_message" in params:
            self.boot_message = params["menu_boot_message"]
        super().validate()
        if "menu_prompt" in params:
            self.selector.prompt = params["menu_prompt"]



# Copyright (C) 2014 Linaro Limited
#
# Author: Neil Williams <neil.williams@linaro.org>
#
# This file is part of LAVA Dispatcher.
#
# LAVA Dispatcher is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LAVA Dispatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along
# with this program; if not, see <http://www.gnu.org/licenses>.

# List just the subclasses supported for this base strategy
# imported by the parser to populate the list of subclasses.

# pylint: disable=unused-import

from lava_dispatcher.actions.boot import SecondaryShell
from lava_dispatcher.actions.boot.bootloader import BootBootloader
from lava_dispatcher.actions.boot.cmsis_dap import CMSIS
from lava_dispatcher.actions.boot.depthcharge import Depthcharge
from lava_dispatcher.actions.boot.dfu import DFU
from lava_dispatcher.actions.boot.docker import BootDocker
from lava_dispatcher.actions.boot.fastboot import BootFastboot
from lava_dispatcher.actions.boot.gdb import GDB
from lava_dispatcher.actions.boot.grub import Grub, GrubSequence
from lava_dispatcher.actions.boot.iso import BootIsoInstaller
from lava_dispatcher.actions.boot.ipxe import IPXE
from lava_dispatcher.actions.boot.kexec import BootKExec
from lava_dispatcher.actions.boot.lxc import BootLxc
from lava_dispatcher.actions.boot.minimal import Minimal
from lava_dispatcher.actions.boot.pyocd import PyOCD
from lava_dispatcher.actions.boot.qemu import BootQEMU
from lava_dispatcher.actions.boot.ssh import SshLogin, Schroot
from lava_dispatcher.actions.boot.u_boot import UBoot
from lava_dispatcher.actions.boot.uefi import UefiShell
from lava_dispatcher.actions.boot.uefi_menu import UefiMenu
from lava_dispatcher.actions.boot.recovery import RecoveryBoot



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-06-24 08:49
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0019_extendeduser")]

    operations = [
        migrations.AlterUniqueTogether(
            name="notificationrecipient",
            unique_together=set([("user", "notification", "method")]),
        )
    ]
